##############
# Drawing truncated normal rvs according to "R Programs for Computing Truncated Distributions" by Saralees Nadarajah and
# Samuel Kotz (2006)


     #this function computes (5) for given spec, a & b
    qtrunc <- function(p, spec, a = -Inf, b = Inf, ...)
    {
        tt <- p
        G <- get(paste("p", spec, sep = ""), mode = "function")
        Gin <- get(paste("q", spec, sep = ""), mode = "function")
        tt <- Gin(G(a, ...) + p*(G(b, ...) - G(a, ...)), ...)
        return(tt)
    }

    #this function computes (6) for given n, spec, a & b
    rtrunc <- function(n, spec, a = -Inf, b = Inf, ...)
    {
        x <- u <- runif(n, min = 0, max = 1)
        x <- qtrunc(u, spec, a = a, b = b,...)
        return(x)
    }

########################
# Dummy function for the specification of spatial effects in a formula
  #sp =function(object,data) return(data$object)
  sp =function(object) return(data$object)

########################
# Function interpret.gam of package mgcv is minimally changed to incorporate spatial effects specified by sp(.). see package mgcv.
  read.formula<-function (gf)
  { p.env<-environment(gf) # environment of formula
    tf<-terms.formula(gf,specials=c("s","sp")) # specials attribute indicates which terms are smooth

    terms<-attr(tf,"term.labels") # labels of the model terms
    nt<-length(terms) # how many terms?

    if (attr(tf,"response")>0)  # start the replacement formulae
    { response<-as.character(attr(tf,"variables")[2])
      pf<-rf<-paste(response,"~",sep="")
    }
    else pf<-rf<-"~"
    sp<-attr(tf,"specials")$s     # array of indices of smooth terms
    tp<-attr(tf,"specials")$sp    # indices of spatial terms
    vtab <- attr(tf,"factors") # cross tabulation of vars to terms
    if (length(sp)>0) for (i in 1:length(sp)) {
      ind <- (1:nt)[as.logical(vtab[sp[i],])]
      sp[i] <- ind # the term that smooth relates to
    }
    if (length(tp)>0) for (i in 1:length(tp)) {
      ind <- (1:nt)[as.logical(vtab[tp[i],])]
      tp[i] <- ind # the term that smooth relates to
    } ## re-referencing is complete

    ns<-length(sp)+length(tp) # number of smooths
    k<-kt<-ks<-kp<-1 # counters for terms in the 2 formulae
    len.sp <- length(sp)
    len.tp <- length(tp)

    smooth.spec<-list()
    if (nt)
    for (i in 1:nt) # work through all terms
    { if (k<=ns&&((ks<=len.sp&&sp[ks]==i)||(kt<=len.tp&&tp[kt]==i)))
      { st<-eval(parse(text=terms[i]),envir=p.env)
        smooth.spec[[k]]<-st
        if (ks<=len.sp&&sp[ks]==i) ks<-ks+1  # counts s() terms
        else kt<-kt+1              # counts sp() terms
        k<-k+1     # counts smooth terms
      } else          # parametric
      { if (kp>1) pf<-paste(pf,"+",terms[i],sep="") # add to parametric formula
        else pf<-paste(pf,terms[i],sep="")
        kp<-kp+1    # counts parametric terms
      }
    }

    if (attr(tf,"intercept")==0)
    { pf<-paste(pf,"-1",sep="")
      if (kp>1) pfok<-1 else pfok<-0
    } else {
      pfok<-1;if (kp==1) {
      pf<-paste(pf,"1"); 
      }
    }
  ret<-list(pf=as.formula(pf,p.env),pfok=pfok,smooth.spec=smooth.spec, response=response)
  class(ret)<-"split.gam.formula"
  ret
  }


########################
#Construct B-spline basis matrix
  bspline = function(covariate, numKnots, knotchoice, degree){
    if(knotchoice=="equidistant")
      knots = seq(min(covariate), max(covariate), length=numKnots)
#      else if(knotchoice=="quantiles")
#        knots = quantile(covariate, probs=seq(0, 1, length=numKnots), names=F)
    else stop("Unknown knotchoice.\n")
    knots = knots[-c(1,numKnots)]
    X = bs(x=covariate, knots=knots, degree=degree, intercept=TRUE)
#    n=length(covariate)
 #   X=t(apply(X,1,function(x) x-colSums(X)/n))
    return(X)
  }


########################
# Create Markov random field penalty. 
  pen.mrf = function(object){
    #    K.mrf = read.gra(file)
    regions = attributes(object)$dimnames[[1]]
    return(list(K.mrf=object, regions=regions))
  }
#  pen.mrf = function(file){
#    mapinfo = scan("nigeria.gra")
#    S = mapinfo[1]
#    mapinfo = mapinfo[-1]
#    regions = rep(0,S)
#    K.mrf = matrix(0,S,S)
#    for(i in 1:S){
#        regions[i] = mapinfo[1]
#        mapinfo = mapinfo[-1]
#        nrn = mapinfo[1]
#        mapinfo = mapinfo[-1]
#        K.mrf[i,i] <- nrn
#        for(j in 1:nrn) K.mrf[i,mapinfo[j]+1] = -1
#        mapinfo = mapinfo[-(1:nrn)]
#      }
#    return(list(K.mrf=K.mrf, regions=regions))
#  }
#


###############
# Plotting of sampling paths of parametric effects      #ONLY INTERNAL
  plotPaths=function(sample, names, numBurnIn=0, output="window",style="together",...){
    if (output!="window") pdf(output)
    if (style=="together"){
      if (output=="window")  dev.new()
      matplot(sample, type="l",
              main="Sampling paths of parametric covariates",
              xlab="iteration", ylab="Samples",col=1:dim(sample)[2],...)
      # Legend
        legend(x="topright", legend=names, lty=1,col=1:dim(sample)[2])
      # Line at the end of the burn-in period
        abline(v=numBurnIn, col=grey(0.8))
    }
    if (style=="separate"){
      for (j in 1:ncol(sample)){
        if (output=="window") dev.new()
        plot(sample[,j],type="l", xlab="iteration", ylab=names[j],...)
        # Line at the end of the burn-in period
          abline(v=numBurnIn, col=grey(0.8))
      }
    }
  if (output!="window") dev.off()
  }






print.bayesIV=function(x,...){
if (class(x)[2]=="ssm"){
cat("ssm object, created:",x$date,"\n")
  cat("\nFormulas: \n")
  cat(" Selection equation: ")
  print(x$formulas$selection)
  cat(" Outcome equation:   ")
  print(x$formulas$outcome)
  cat("\n",length(x$responses$z)," observations (",sum(x$responses$z==0)," censored and ",sum(x$responses$z==1)," observed)" , sep = "")
  cat("\nNumber of samples discarded (burn-in): ",x$numBurnIn,"; Number of Samples used: ",floor(x$numSamples/x$thin)," of ",x$numSamples," Samples (thinning=",x$thin,")\n", sep = "")
}
else{
cat(class(x)[2],"object, created:",x$date,"\n")
  cat("\nFormulas: \n")
  cat(" First equation: ")
  print(x$formulas$first)
  cat(" Second equation:   ")
  print(x$formulas$second)
  cat("\n",length(x$responses[[1]])," observations" , sep = "")
  cat("\nNumber of samples discarded (burn-in): ",x$numBurnIn,"; Number of Samples used: ",floor(x$numSamples/x$thin)," of ",x$numSamples," Samples (thinning=",x$thin,")\n", sep = "")
}

}










