##############################################################
#                                                            #
# Requires packages mvtnorm and splines. Package BayesX 0.2  #
# is required if spatial effects are included.               #
#                                                            #
##############################################################

bayesIV =
function(first, second, random=list(NULL,NULL),data=list(),numBurnIn=2000, numSamples=20000, thin=20
, contrasts.first =NULL, contrasts.second =NULL
, graph= NULL
, x.pred=NULL
, numKnots=40, knotchoice="equidistant", degree=3, rw.order=2
, density.method="jara"
, priorDPM =
  list(
    # fixed parameters in G0 (Conley+Jara)
       tau.Sigma=.0036,
      s.Sigma=3, S.Sigma=0.2*diag(2),
    # hyperpriors on parameters in G0 (Jara)
#      m.mu=NULL,s2.mu=NULL,
      a.Sigma=NULL,b.Sigma=NULL,
      s.S.Sigma=NULL,S.S.Sigma=NULL,
    # parameters for alpha discretely distributed (Conley)
      power=.8,  maxuniq=200,  Istarmin=1, Istarmax=NULL, ngrid.alpha=NULL,
    # parameters for alpha Gamma distributed (Jara)
      a.alpha=2,b.alpha=2
  )
, prior.random= c(0.001,0.001)
, prior.tau=c(0.001,0.001), prior.tau.sp=c(0.001,0.001)
, grid.density=NULL
, ngrid.density=1000
, seed=NULL
, scbs=F,scb.center=T,scb.level=0.95 #,scb.method="TK"
, record.npcoef=F, return.samples=T, return.f=T
, mgcv=T, plotpaths=F #, plotpaths.style="separate"
, progressBar=T
, method="chol"){

 SCALE=T
 priorDPM$mu0=c(0,0)

################################################
result=list()

date=format(Sys.time(), "%Y-%m-%d--%Hh%Mm%Ss")
result$date=date
result$priorDPM=priorDPM

p0=proc.time()
################################################
# Read formulas
  #------------------
  #first equation
    # Interpret formula of first equation
      np=interpret.gam(first)
    # recognize on which location in the formula there is a s() (for smooth functions) or sp() (for spatial effects)
      tf.first = terms.formula(first, specials = c("s", "sp"))
      sp_first = attr(tf.first, "specials")$sp
    # Number of nonparametric effects
      numSplines_first = length(np$smooth.spec)
      numSpatial_first = length(sp_first)
    # get smooth parameters and 'by' variables
      npvars.first= byvars.first= numeric(numSplines_first)
      if (numSplines_first>0){
        for (j in 1:(numSplines_first)){
          npvars.first[j]=np$smooth.spec[[j]]$term
          byvars.first[j]=np$smooth.spec[[j]]$by
        }
      }
    # spatial effects
 #     spvar.first=all.vars(first, unique = FALSE)[-which(all.vars(first, unique = FALSE) %in% byvars.first[byvars.first!="NA"])][sp_first]
    spvar.first=all.vars(first, unique = FALSE)[sp_first]
  #--------------------
  # second equation
    # interpret formula of second equation
      np=interpret.gam(second)
    # recognize on which location in the formula there is a s() (for smooth functions) or sp() (for spatial effects)
      tf.second = terms.formula(second, specials = c("s", "sp"))
      sp_second = attr(tf.second, "specials")$sp
     # Number of nonparametric effects
      numSplines_second = length(np$smooth.spec)
      numSpatial_second = length(sp_second)
    # get smooth parameters and by variables
      npvars.second= byvars.second= numeric(numSplines_second)
      if (numSplines_second>0){
        for (j in 1:(numSplines_second)){
          npvars.second[j]=np$smooth.spec[[j]]$term
          byvars.second[j]=np$smooth.spec[[j]]$by
        }
      }
    # spatial effects
 #     spvar.second=all.vars(second, unique = FALSE)[-which(all.vars(second, unique = FALSE) %in% byvars.second[byvars.second!="NA"])][sp_second]
      spvar.second=all.vars(second, unique = FALSE)[sp_second]
  #------------------
  # Combined
    # Auxillary variables -> for spatial effects no varying coefficients are implemented, these variables are only needed as wild-cards in 'byvars'
      if (numSpatial_first>0) byvarSp.first_aux="NA"  else byvarSp.first_aux= numeric(0)
      if (numSpatial_second>0) byvarSp.second_aux="NA"  else byvarSp.second_aux= numeric(0)

    # Vector of nonparametric effects including spatial effects if included  (note the order of first&second&spatial!)
      npvars=c(npvars.first,spvar.first,npvars.second,spvar.second)
      byvars=c(byvars.first, byvarSp.first_aux, byvars.second, byvarSp.second_aux)

################################################
# Spatial Data
  # Estimate spatial effects?
    geo=FALSE
    if ((is.null(graph)==F) & (numSpatial_second>0 | numSpatial_first>0)) geo=TRUE
  # Read spatial data
    if (geo){
      pmat=K.mrf=regions=numRegions=list()
      if (numSpatial_first>0){
        pmat[[1]] =pen.mrf(graph)
        regions[[1]]=pmat[[1]]$regions
        numRegions[[1]]=length(regions[[1]])
        # penalty matrix
          K.mrf[[1]]=pmat[[1]]$K.mrf
      }
      if (numSpatial_second>0){
        pmat[[2]] =pen.mrf(graph)
        regions[[2]]=pmat[[2]]$regions
        numRegions[[2]]=length(regions[[2]])
        # penalty matrix
          K.mrf[[2]]=pmat[[2]]$K.mrf
      }
    }
################################################
# Model matrices and responses
  #=============================================================================
  # Responses and model matrices of parametric covariates

    # Model formula for the strictly parametric part of the model.
      firstPar= read.formula(first)$pf
      secondPar= read.formula(second)$pf
    # Read formulas and convert to model frames
      frame.firstPar= model.frame(firstPar,data=data)
      frame.secondPar= model.frame(secondPar, data=data)
    # Responses
      response = list()
      response[[1]] = model.response(frame.firstPar)
      response[[2]] = model.response(frame.secondPar)#[response[[1]] ==1]
      response.first=all.vars(first)[1]
      response.second=all.vars(second)[1]
    # Design matrices of parametric effects
      V = list()
#      V[[1]] = model.matrix(firstPar, frame.firstPar,contrasts.arg=contrasts.first)[,-1,drop=F]
#      V[[2]] = model.matrix(secondPar, frame.secondPar,contrasts.arg=contrasts.second)[,-1,drop=F]#[response[[1]]==1, ,drop=F]
      V[[1]] = model.matrix(firstPar, frame.firstPar,contrasts.arg=contrasts.first)
      V[[2]] = model.matrix(secondPar, frame.secondPar,contrasts.arg=contrasts.second)

  #=============================================================================
  # Design matrices and initialisation for random effects
    REyes=c(!is.null(random[[1]]),!is.null(random[[2]]))

    result$Z=a.sig_delta=b.sig_delta=sig_delta2=delta=list()
    for (r in 1:2){
      if (REyes[r]){
        m=length(unique(random[[r]]))
        result$Z[[r]]=matrix(0,ncol=m,nrow=length(response[[r]]))
        for (i in 1:m) result$Z[[r]][(random[[r]]==unique(random[[r]])[i]),i]=1
      a.sig_delta_0 =prior.random[1]
      b.sig_delta_0 =prior.random[2]
      a.sig_delta[[r]]=a.sig_delta_0+m/2
      delta[[r]] = matrix(data=0, nrow=(numBurnIn+numSamples+1), ncol=m)   #dim(V[[2]])[2]=length(gamma[[2]])
      }else{ result$Z[[r]]=0;delta[[r]] =  matrix(data=0, nrow=(numBurnIn+numSamples+1), ncol=1)  }  #  delta[[r]] =
      sig_delta2[[r]] = matrix(data=0, nrow=(numBurnIn+numSamples+1), ncol=1)
    }

  #=============================================================================
  # Scaling
    if (SCALE){
      V.unscaled=V
      scale1=sqrt(var(response[[1]]))
      scale2=sqrt(var(response[[2]]))
      mu1=mean(response[[1]])
      mu2=mean(response[[2]])
      response[[1]] = scale(response[[1]],scale=T,center=T)
      response[[2]] = scale(response[[2]],scale=T,center=T)

      
      #if there are only two distinct values, treat as dummy (factor)
        fid=(apply(V[[1]],2, function(x) length(unique(x)))==2)
        for (i in 1:length(fid)){ if (fid[i]) V[[1]][,i]=as.factor(V[[1]][,i]) }
        fid=(apply(V[[2]],2, function(x) length(unique(x)))==2)
        for (i in 1:length(fid)){ if (fid[i]) V[[2]][,i]=as.factor(V[[2]][,i]) }
      
      
      # only center not factor variables and exclude intercept
        fac=logical()
      #  print(V)
      #  print(length(attr(V[[1]],"contrasts")))
      #  print(         attr(terms(firstPar),"term.labels"))
        for (i in 1:length(attr(terms(firstPar),"term.labels"))){
         fac[i]=try((attr(V[[1]],"contrasts")[[attr(terms(firstPar),"term.labels")[i]]]=="contr.treatment") )
      #   if (!is.logical(fac[i])){print(fac[i]); fac[i]=FALSE  }
        }
        fac[is.na(fac)]=FALSE
        fac=c(T,fac)
      #  print(fac)
        if (!is.null(dim(V[[1]][,!fac]))) V[[1]][,!fac]= (apply(V[[1]][,!fac],2,function(x) x-mean(x)))
      #  V[[1]]=V[[1]][,-1,drop=F]

        fac=logical()
        for (i in 1:length(attr(terms(secondPar),"term.labels")))fac[i]=(attr(V[[2]],"contrasts")[[attr(terms(secondPar),"term.labels")[i]]]=="contr.treatment")
        fac[is.na(fac)]=FALSE
        fac=c(T,fac)
 #         print(fac)
      if (!is.null(dim(V[[2]][,!fac]))) V[[2]][,!fac]= (apply(V[[2]][,!fac],2,function(x) x-mean(x)))
      #  V[[2]]=V[[2]][,-1,drop=F]

      if (!is.null(as.data.frame(V[[2]])[[response.first]])) V[[2]][,response.first] =response[[1]]
    } else {scale1=scale2=1;mu1=mu2=0}
    scalevec=c(scale1,scale2)
    centervec=c(mu1,mu2)

    # Name vector of parametric effects including Intercept if included
      g_first=dimnames(V[[1]])[[2]]
      g_second=dimnames(V[[2]])[[2]]

  #=============================================================================
  # List of the nonparametric covariates
    npcovariates = lapply(as.list(npvars), function(x) data[[x]])
    byvars.list  = lapply(as.list(byvars), function(x) data[[x]])      # List of variables given by a 'by=' argument for varying coefficients
    if (length(npvars)>0){
      names(npcovariates)=npvars
      npcovariates.unscaled=npcovariates

      if (numSpatial_first==1) npcovariates.unscaled[[spvar.first]]=as.numeric(as.character(npcovariates.unscaled[[spvar.first]]))
      if (numSpatial_second==1) npcovariates.unscaled[[spvar.second]]=as.numeric(as.character(npcovariates.unscaled[[spvar.second]]))
      if (SCALE){
#####
#
#
#was ist mit byvars?
#
#
###
        npcovariates=lapply(npcovariates,function(x) scale(as.numeric(x),scale=F))
        if (!is.null(npcovariates[[response.first]])){whichendo=which(npvars==response.first); npcovariates[[response.first]] =response[[1]]}else whichendo=NULL
      }else whichendo=NULL
      for (j in 1:length(npvars)) if (byvars[j]!="NA") names(byvars.list)[j] = byvars[j]
    }
  #-----------------------------------------------------------------------------
  # Model matrices of the nonparametric covariates
    X=  Xby= X.pred=f.pred.sum= list()
    if (length(npvars) > 0){
      # Prediction on a given grid if specified
        if (is.null(x.pred)) {
          x.pred=list()
          length(x.pred)=length(npvars)
        }
        else 
        if (!is.list(x.pred)){
            temp=list()
            for (i in 1:length(npvars)) temp[[i]]=x.pred
            x.pred=temp
            rm(temp)
        }
      for (j in 1:length(npvars)){    #first first, then  second equation
        # Write design matrix of spatial covariate
          if (numSpatial_first>0 & j==numSplines_first+1){
            X[[j]] = matrix(0,length(npcovariates.unscaled[[j]]),numRegions[[1]])
              for(i in 1:numRegions[[1]])
                X[[j]][which(npcovariates.unscaled[[j]]==regions[[1]][i]),i] = 1     # 1 if y_i1 was observed in region s
            if (!is.null(x.pred[[j]])){
              X.pred[[j]] = matrix(0,length(x.pred[[j]]),numRegions[[1]])
              f.pred.sum[[j]]=0
                for(i in 1:numRegions[[1]])
                  X.pred[[j]][which(x.pred[[j]]==regions[[1]][i]),i] = 1
            }
          }
      if (numSpatial_second>0 &  j==numSplines_first+numSpatial_first+numSplines_second+1){
            X[[j]] = matrix(0,length(npcovariates.unscaled[[j]]),numRegions[[2]])
              for(i in 1:numRegions[[2]])
                X[[j]][which(npcovariates.unscaled[[j]]==regions[[2]][i]),i] = 1      # 1 if y_i2 was observed in region s
            if (!is.null(x.pred[[j]])){
              X.pred[[j]] = matrix(0,length(x.pred[[j]]),numRegions[[2]])
              f.pred.sum[[j]]=0
                for(i in 1:numRegions[[2]])
                  X.pred[[j]][which(x.pred[[j]]==regions[[2]][i]),i] = 1
            }              
          }
        # write B-spline basis design matrix of nonparametric effects into list element j
          if (j<=numSplines_first | (j>numSplines_first+numSpatial_first&j<=numSplines_first+numSpatial_first+numSplines_second)){
            X[[j]]=bspline(npcovariates[[j]],numKnots=numKnots,knotchoice=knotchoice,degree=degree)
            if (!is.null(x.pred[[j]])){
              X.pred[[j]]=bspline(x.pred[[j]],numKnots=numKnots,knotchoice=knotchoice,degree=degree)
              f.pred.sum[[j]]=0
            }
          }
      
        if (byvars[j]!="NA"){
          Xby[[j]]=X[[j]]
          X[[j]]= diag(byvars.list[[j]]) %*% X[[j]]
        }
      }
    }

#%%%%%%%%%%%%%%%
# discard unnecessary matrices
  rm(frame.firstPar, frame.secondPar)
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

################################################
# Some error search:
  # Intercept: Either 0, indicating no intercept is to be fit, or 1 indicating that an intercept is to be fit
    int_first = attr(tf.first,"intercept")
    int_second = attr(tf.second,"intercept")
  if (int_first==0|int_second==0) stop("Intercepts must be given in formulas.")
  if ((int_first==0 & (numSplines_first>0|numSpatial_first>0))|(int_second==0 & (numSplines_second>0|numSpatial_second>0))) stop("When nonparametric effects are included, an intercept is required.")
  if ((numSpatial_first>0|numSpatial_second>0) && !geo) stop("Graph not correctly specified")
  if (scbs==T & return.f==F){warning("return.f set to TRUE to allow for SCBs");return.f=T}

##############################################################
# Penalty matrix for smooth functions
  D= diff(diag(numKnots+2),differences=rw.order)
  K.rw=crossprod(D)

##############################################################
# Initialization
  if (!is.null(seed)) set.seed(seed)
  offset= VAR.gamma= mu.gamma= gamma= f.gamma= eta= list()
  Dev =numeric(numBurnIn+numSamples+1)
  gamma[[1]] = matrix(data=0, nrow=(numBurnIn+numSamples+1), ncol=dim(V[[1]])[2])   #dim(V[[1]])[2]=length(gamma[[1]])
  gamma[[2]] = matrix(data=0, nrow=(numBurnIn+numSamples+1), ncol=dim(V[[2]])[2])   #dim(V[[2]])[2]=length(gamma[[2]])

  #***************************************************************************
  # Inizialize betas, f's etc.
    beta= beta.rec= f= f.sum=f.rec= fnr= f.pred=f.pred.rec=list()
    if (length(npvars)>0){
      f[1:length(npvars)]= f.sum[1:length(npvars)]= beta[1:length(npvars)]= 0
      if (!all(unlist(lapply(x.pred,is.null)))) f.pred[1:length(npvars)]= 0
      fnr[1:length(npvars)]=1
      
      for (j in 1:length(npvars)) {
        beta[[j]]= numeric(dim(X[[j]])[2])    #betas have a starting value zero
        if (record.npcoef| byvars[j]!="NA")     #beta is only recorded if desired
          beta.rec[[j]]=matrix(data=0, ncol=dim(X[[j]])[2],nrow=floor(numSamples/thin))
        else beta.rec[[j]]=numeric()
        if (byvars[j]=="NA"){  
          f.rec[[j]]=matrix(data=0, nrow=dim(X[[j]])[1],ncol=floor(numSamples/thin))
          if (!is.null(x.pred[[j]])) f.pred.rec[[j]]=matrix(data=0, nrow=dim(X.pred[[j]])[1],ncol=floor(numSamples/thin))
         }
        else f.rec[[j]]=numeric()
      }
      names(beta.rec)=names(f.rec)=npvars
    }


################################################
# Comperative estimation
  #------------------
  # Auxillary formulas
    #------------------
    # Formulas with all covariates parametric (including those nonparametrically defined)
      fakeformula.first=as.formula(paste(response.first,"~",paste(c(all.vars(firstPar)[-1],npvars.first), collapse="+")))
      fakeformula.second=as.formula(paste(response.second,"~",paste(c(all.vars(secondPar)[-1],npvars.second), collapse="+")))
    #------------------
    # Formulas without spatial effects
      covariates.first=as.vector(attr(tf.first,"term.labels"))
        if (numSpatial_first>0) covariates.first=covariates.first[-(sp_first-1)]
      covariates.second=as.vector(attr(tf.second,"term.labels"))
        if (numSpatial_second>0) covariates.second=covariates.second[-(sp_second-1)]

      mgcvformula.first=as.formula(paste(response.first,"~",paste(covariates.first, collapse="+")))
      mgcvformula.second=as.formula(paste(response.second,"~",paste(covariates.second, collapse="+")))
  #------------------
  # Univariate estimation with mgcv package
    result$f.mgcv=result$f.mgcv.pred=list()
    if (mgcv){
      #cat("\nUnivariate estimation using package mgcv\n")
      if (is.data.frame(data)) datam=data else datam=NULL
      mgcv.second=gam(mgcvformula.second, data=datam)
      mgcv.first=gam(mgcvformula.first, data=datam)
      rm(datam)
      #print(summary(mgcv.first))
      #print(summary(mgcv.second))

      coef.mgcv_first=as.vector(mgcv.first$coefficients)[(1:ncol(V[[1]]))]  #      1+
      coef.mgcv_second=as.vector(mgcv.second$coefficients)[(1:ncol(V[[2]]))]#    1+
      sd.coef.mgcv_first=sqrt(diag(vcov(mgcv.first)))[(1:ncol(V[[1]]))]      #   1+
      sd.coef.mgcv_second=sqrt(diag(vcov(mgcv.second)))[(1:ncol(V[[2]]))]    # 1+
      # values of f
        f.mgcv.first=predict(mgcv.first,type="terms")
        if (length(attr(mgcv.first$pterms,"term.labels"))>0) f.mgcv.first=f.mgcv.first[,-(1:length(attr(mgcv.first$pterms,"term.labels"))) ,drop=F]
        f.mgcv.second=predict(mgcv.second,type="terms")
        if (length(attr(mgcv.second$pterms,"term.labels"))>0) f.mgcv.second=f.mgcv.second[,-(1:length(attr(mgcv.second$pterms,"term.labels"))) ,drop=F]
        # as list
          if (numSplines_first>0) for (j in 1:ncol(f.mgcv.first)) result$f.mgcv[[j]]=f.mgcv.first[,j]
          if (numSplines_second>0) for (j in 1:ncol(f.mgcv.second)) result$f.mgcv[[j+ncol(f.mgcv.first)]]=f.mgcv.second[,j]

        # Prediction on fixed grid
        if ((numSplines_first>0 | numSplines_second>0) & !all(unlist(lapply(x.pred,is.null)))){
          newdata=V[[1]][1:length(x.pred[[1]]),]
          for (j in 1:numSplines_first) newdata=as.data.frame(cbind(newdata, x.pred[[j]]))
          dimnames(newdata)[[2]]=c(g_first,npvars.first[1:numSplines_first])
          pred.mgcv.first=predict(mgcv.first,newdata=newdata,type="terms")
          if (length(attr(mgcv.first$pterms,"term.labels"))>0) pred.mgcv.first=pred.mgcv.first[,-(1:length(attr(mgcv.first$pterms,"term.labels"))) ,drop=F]

          newdata=V[[2]][1:length(x.pred[[numSplines_first+numSpatial_first+1]]),]
          for (j in 1:numSplines_second) newdata=as.data.frame(cbind(newdata, x.pred[[numSplines_first+numSpatial_first+j]]))
          dimnames(newdata)[[2]]=c(g_second,npvars.second[1:numSplines_second])
          pred.mgcv.second=predict(mgcv.second,newdata=newdata,type="terms")
          if (length(attr(mgcv.second$pterms,"term.labels"))>0) pred.mgcv.second=pred.mgcv.second[,-(1:length(attr(mgcv.second$pterms,"term.labels"))) ,drop=F]

          # as list
            if (numSplines_first>0) for (j in 1:ncol(pred.mgcv.first)) result$f.mgcv.pred[[j]]=pred.mgcv.first[,j]
            if (numSplines_second>0) for (j in 1:ncol(pred.mgcv.second)) result$f.mgcv.pred[[j+ncol(pred.mgcv.first)]]=pred.mgcv.second[,j]
        }
      rm(mgcv.first,mgcv.second,f.mgcv.first,f.mgcv.second)
    }
    else{
      coef.mgcv_first=rep(NA,ncol(V[[1]]))
      coef.mgcv_second=rep(NA,ncol(V[[2]]))
      sd.coef.mgcv_first=sd.coef.mgcv_second=NA
    }
    result$coefficients.mgcv=list(first=coef.mgcv_first, second=coef.mgcv_second)
    result$sd.coefficients.mgcv=list(first=sd.coef.mgcv_first,second=sd.coef.mgcv_second)

##############################################################
# Starting values and priors
  # Parametric effects
    # starting value of gamma[[2]] estimated via least squares, starting value of gamma[[1]] estimated via ML
    if (mgcv){
      gamma[[1]][1,]=coef.mgcv_first
      gamma[[2]][1,]=coef.mgcv_second
      result$coefficients.lm_first=(lm(fakeformula.first,data=data)$coef)[(1:ncol(V[[1]]))]  #
      result$coefficients.lm_second=(lm(fakeformula.second,data=data)$coef)[(1:ncol(V[[2]]))]#
    }
    else{
      result$coefficients.lm_first=gamma[[1]][1,]=(lm(fakeformula.first,data=data)$coef)[1:ncol(V[[1]])]#[-1]#
      result$coefficients.lm_second=gamma[[2]][1,]=(lm(fakeformula.second,data=data)$coef)[1:ncol(V[[2]])]#[-1]#
    }
    gamma[[1]][1,] = gamma[[1]][1,]/scale1
    gamma[[2]][1,] = gamma[[2]][1,]/scale2
    f.gamma[[1]]= V[[1]] %*% gamma[[1]][1,]
    f.gamma[[2]]= V[[2]] %*% gamma[[2]][1,]

  # Nonparametric effects
    # see initialize betas ==0

  # Starting values of the predictors
    # just linear effects, starting values of nonpar effects ==0
      eta[[1]]   = V[[1]] %*% gamma[[1]][1,]
      eta[[2]]   = V[[2]] %*% gamma[[2]][1,]  #    +(lm(fakeformula.second,data=data)$coef)[1]

  # tau2
    a.tau.sp=list()
    a.tau.rw_0 = prior.tau[1]
    b.tau.rw_0 = prior.tau[2]
    a.tau.sp_0 = prior.tau.sp[1]
    b.tau.sp_0 = prior.tau.sp[2]
    # the first parameter remains constant, no updating necessary
      a.tau.rw = a.tau.rw_0 + qr(K.rw)$rank/2
      if (geo) {
        if (numSpatial_first>0) a.tau.sp[[1]] = a.tau.sp_0 + qr(K.mrf[[1]])$rank/2
        if (numSpatial_second>0) a.tau.sp[[2]] = a.tau.sp_0 + qr(K.mrf[[2]])$rank/2
      }


    n=length(response[[1]])

# DPM stuff
    result$density=list(fun1=0,fun2=0,f=0,sigma1=0,sigma2=0,sigma12=0,mui1=0,mui2=0)
  if (density.method=="conley"){
    if (is.null(priorDPM$Istarmax)) priorDPM$Istarmax=floor(.1*n)
    if (is.null(priorDPM$ngrid.alpha)) gridsize=floor(n/10) else gridsize=priorDPM$ngrid.alpha

    gammaDP= .5772156649015328606
    alphamin=exp(digamma(priorDPM$Istarmin)-log(gammaDP+log(n)))
    alphamax=exp(digamma(priorDPM$Istarmax)-log(gammaDP+log(n)))
    Prioralpha=list(n=n,alphamin=alphamin,alphamax=alphamax,power=priorDPM$power)

    # lambda=list(mubar=c(0,0),Amu=.2,nu=3.4,V=1.7*diag(2))
    lambda=list(mubar=priorDPM$mu0,Amu=priorDPM$tau.Sigma,nu=priorDPM$s.Sigma ,V=priorDPM$S.Sigma)
    onecomp=list(mu=c(0,0),rooti=diag(2))
    theta=vector("list",n)
    for(i in 1:n) {theta[[i]]=onecomp}
    comps=unique(theta)
    ncomp=length(comps)
    indic=double(n)
    for(j in 1:ncomp) indic[which(sapply(theta,identical,comps[[j]]))]=j

    alpha=1 #starting value for alpha

    Istardraw=alphadraw=numeric()
    thetaNp1draw =list()
    cat("Density estimation using DPM specification according to Conley et al. (2008)\n\n")
    cat("Parameters of Prior on Dirichlet Process precision parameter (alpha):",fill=TRUE)
    cat(" alphamin= ",alphamin," alphamax= ",alphamax," power=",priorDPM$power,fill=TRUE)
    cat(" alpha values correspond to Istarmin = ",priorDPM$Istarmin," Istarmax = ",priorDPM$Istarmax,fill=TRUE)
    cat("\nParameters of priors on base distribution:",fill=T)
    print(list(mu0=priorDPM$mu0,tau.Sigma=priorDPM$tau.Sigma,s.Sigma=priorDPM$s.Sigma ,S.Sigma=priorDPM$S.Sigma))
    cat("Maximum number of unique mixture components= ",priorDPM$maxuniq,fill=TRUE)
    cat("gridsize for alpha draws= ",gridsize,fill=TRUE)
    cat("SCALE data= ",SCALE,"\n\n")
    #mbg=Abg=list()
    #mbg[[1]]=c(rep(0,dim(V[[1]])[[2]]))
    #Abg[[1]]=diag(0.01,dim(V[[1]])[[2]])
    #mbg[[2]]=c(rep(0,dim(V[[2]])[[2]]))
    #Abg[[2]]=diag(0.01,dim(V[[2]])[[2]])
  }
  else if (density.method=="jara"){
    cat("Density estimation using DPM specification using DPpackage (Jara)",fill=TRUE)
    fitDPM=list(temp=1)
    priorDPM=list(
      # fixed parameters in G0
        m1=priorDPM$mu0, k0=priorDPM$tau.Sigma ,
        nu1 = priorDPM$s.Sigma, psiinv1=priorDPM$S.Sigma,
      # hyperpriors on parameters in G0
        m2 = priorDPM$m.mu, s2 = priorDPM$s2.mu,
        tau1 = priorDPM$a.Sigma, tau2 = priorDPM$b.Sigma,
        nu2 = priorDPM$s.S.Sigma,  psiinv2 = priorDPM$S.S.Sigma,
      # parameters for alpha Gamma distributed
        a0 = priorDPM$a.alpha, b0 = priorDPM$b.alpha
    )
#    if (!is.null(priorDPM$tau1)) priorDPM$tau1=2*priorDPM$tau1
#    if (!is.null(priorDPM$tau2)) priorDPM$tau1=2*priorDPM$tau2
    if (is.null(grid.density)){
      ngrid.density <- as.integer(sqrt(ngrid.density))
      result$density$grid1 <- seq(min(response[[1]])-0.5*sqrt(var(response[[1]])),max(response[[1]])+0.5*sqrt(var(response[[1]])),length=ngrid.density)
      result$density$grid2 <- seq(min(response[[2]])-0.5*sqrt(var(response[[2]])),max(response[[2]])+0.5*sqrt(var(response[[2]])),length=ngrid.density)
      grid.density=cbind(result$density$grid1,result$density$grid2)
    }
  }

result$int.sample=matrix(data=0, nrow=floor(numSamples/thin), ncol=2)
result$yhat.sample=list(first= matrix(data=0, nrow=n,ncol=floor(numSamples/thin)),second= matrix(data=0, nrow=n,ncol=floor(numSamples/thin)))

result$yhat=matrix(0,n,2)
result$cmean=result$cmeanB=result$cmeanB2=result$cmean2=result$eps=result$eps2=list(0,0)
intnr=1
#SigmaMat=cbind(matrix(1,n,2),rep(0,n))
#mui=matrix(0,2,n)
#m.mui=c(0,0)
############
#  GIBBS   #
############
p8=proc.time()
if (progressBar) pb = txtProgressBar(style=3)
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
# Start of loop
for ( i in 2:(1+numBurnIn+numSamples) ){
  if (progressBar & i>51 & i%%100==0) setTxtProgressBar(pb, i/(1+numBurnIn+numSamples))
   #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  # DP process stuff- theta | lambda
    Err = cbind(response[[1]]-eta[[1]]-REyes[1]*(result$Z[[1]]%*%delta[[1]][i-1,])[,,drop=T],response[[2]]-eta[[2]]-REyes[2]*(result$Z[[2]]%*%delta[[2]][i-1,])[,,drop=T])
    if (density.method=="conley"){
      q0v = q0(Err,lambda,NULL)
      DPout=rthetaDPbayesIV(maxuniq=priorDPM$maxuniq,alpha=alpha,lambda=lambda,Prioralpha=Prioralpha,theta=theta,
                     thetaStar=comps,indic=indic,q0v=q0v,y=Err,gridsize=gridsize)
      theta=DPout$theta
      alpha=DPout$alpha
      Istar=DPout$Istar
      Istardraw[i]=Istar
      alphadraw[i]=alpha
      if (i>(numBurnIn+1) & (i-(numBurnIn+1))%%thin==0) thetaNp1draw=c(thetaNp1draw,list(list(DPout$thetaNp1)))

      indic=matrix(DPout$indic)
      comps=DPout$thetaStar
      ncomp=length(comps)

      mud=lapply(comps, function(x) return(x[[1]]))
      
      mui=matrix(0,ncol=n,nrow=2)
      for (k in 1:length(unique(comps))) mui[,indic==k]=mud[[k]]
      Sigma= lapply(comps, function(x) crossprod(backsolve(x[[2]],diag(2))))

      #centering
        m.mui=rowMeans(mui)
        mui=mui-m.mui
         comps=lapply(comps,function(x) return(list(mu=x[[1]]-m.mui,rooti=x[[2]])))

     if (i>(numBurnIn+1) & (i-(numBurnIn+1))%%thin==0){
       result$density$mui1=result$density$mui1+mui[1,]
       result$density$mui2=result$density$mui2+mui[2,]
       }


      }
    else if (density.method=="jara"){
      do=TRUE
      while (do){                                             # just sample again in case of an error (not pd matrix)
        if (i>(numBurnIn+1) & (i-(numBurnIn+1))%%thin==0){
          fitDPM1 <- try(DPdensity2(y = Err,
          prior = priorDPM,mcmc = list(nburn = 0, nsave = 1,nskip = 1, ndisplay = 2),
          state = fitDPM$state,status = is.null(fitDPM$state),na.action = na.omit,method="neal" ,grid=grid.density))   #     method=      no-gaps  "neal"
        }
        else{
          fitDPM1 <- try(DPdensity2(y = Err,
          prior = priorDPM,mcmc = list(nburn = 0, nsave = 1,nskip = 1, ndisplay = 2),
          state = fitDPM$state,status = is.null(fitDPM$state),na.action = na.omit,method="neal" ,ngrid=4))   #     method=      no-gaps  "neal"
        }
        if (!is.character(fitDPM1)) do=FALSE        #    else cat("\nWarning: \n")
      }
      fitDPM=fitDPM1
      DPmat=matrix(fitDPM$save.state$randsave[,-c((length(fitDPM$save.state$randsave)-6):length(fitDPM$save.state$randsave))],ncol=5,byrow=T)
      SigmaMat=DPmat[,c(3,5,4)]
      mui=t(DPmat[,1:2])

      m.mui=rowMeans(mui)
      fitDPM$state$muclus[1:fitDPM$state$ncluster,]=  t( t(fitDPM$state$muclus[1:fitDPM$state$ncluster,,drop=F])-m.mui)

      mui=mui-m.mui

##for (r in 1:2){
##  mui[r,]=mui[r,]-m.mui[r]
##
## fitDPM$state$muclus[fitDPM$state$muclus[,1]!=0,r]=  ((fitDPM$state$muclus[fitDPM$state$muclus[,1]!=0,r])-m.mui[r])
##  fitDPM$state$muclus[1:fitDPM$state$ncluster,r]=  ((fitDPM$state$muclus[1:fitDPM$state$ncluster,r])-m.mui[r])
##}
##m.mui=c(0,0)

      if (i>(numBurnIn+1) & (i-(numBurnIn+1))%%thin==0){
       result$density$mui1=result$density$mui1+mui[1,]
       result$density$mui2=result$density$mui2+mui[2,]
       result$density$sigma1=result$density$sigma1+SigmaMat[,1]
       result$density$sigma2=result$density$sigma2+SigmaMat[,2]
       result$density$sigma12=result$density$sigma12+SigmaMat[,3]
       result$density$fun1=result$density$fun1+fitDPM$fun1
       result$density$fun2=result$density$fun2+fitDPM$fun2
       result$density$f=result$density$f+fitDPM$f
      }

    }

  for (r in 1:2){
  # extract conditional variances etc.
      if (density.method=="conley"){
        cond.var= lapply(Sigma,function(x){ (x[r,r]-(x[1,2]^2/x[3-r,3-r]))})
        sg=lapply(Sigma, function(x) return(x[1,2]/x[3-r,3-r]))
        si=cond.vari=numeric(n)
        for (k in 1:length(unique(comps))){si[indic==k]=sg[[k]]; cond.vari[indic==k]=cond.var[[k]] }
      }
      else if (density.method=="jara"){
        cond.vari= SigmaMat[,r]-SigmaMat[,3]^2/SigmaMat[,3-r]
        si= SigmaMat[,3]/SigmaMat[,3-r]
      }



#if (r==2) mui[3-r,]=mui[3-r,]-m.mui[3-r]
#if (r==1)
#etaplus=eta[[3-r]]+ mui[3-r,]
#if (r==2) etaplus=eta[[3-r]]-gamma[[3-r]][i,1]+gamma[[3-r]][i-1,1]+ mui[3-r,]


     offset= si*((response[[3-r]]-eta[[3-r]])-mui[3-r,]-REyes[3-r]*(result$Z[[3-r]]%*%delta[[3-r]][i-1,])[,,drop=T])    #
#       offset= si*(response[[3-r]]-etaplus-REyes[3-r]*(result$Z[[3-r]]%*%delta[[3-r]][i-1,])[,,drop=T])    #

  #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  # Estimation of the coefficients

    # conditional means, intercepts and yhats
      if (i>(numBurnIn+1) & (i-(numBurnIn+1))%%thin==0) {
#        result$eps[[r]]=result$eps[[r]]+  response[[r]]-(eta[[r]]+REyes[r]*(result$Z[[r]]%*%delta[[r]][i-1,])[,,drop=T])
#        result$eps2[[r]]=result$eps2[[r]]+  response[[r]]-(eta[[r]]+mui[r,]+REyes[r]*(result$Z[[r]]%*%delta[[r]][i-1,])[,,drop=T])

#        result$cmean[[r]]=result$cmean[[r]] + (mui[r,]*scalevec[r] + si*scalevec[3-r]*((response[[3-r]]-eta[[3-r]])-mui[3-r,]-REyes[3-r]*(result$Z[[3-r]]%*%delta[[3-r]][i-1,])[,,drop=T]))
#        result$cmean2[[r]]=result$cmean2[[r]] + (mui[r,] + si*((response[[3-r]]-eta[[3-r]])-mui[3-r,]-REyes[3-r]*(result$Z[[3-r]]%*%delta[[3-r]][i-1,])[,,drop=T]))
#        result$cmeanB[[r]]=result$cmeanB[[r]] +centervec[r]/scalevec[r] + mean(eta[[r]])+mui[r,] + si*(response[[3-r]]-eta[[3-r]]-mui[3-r,]-REyes[3-r]*(result$Z[[3-r]]%*%delta[[3-r]][i-1,])[,,drop=T])
#        result$cmeanB2[[r]]=result$cmeanB2[[r]] +centervec[r] + mean(eta[[r]])*scalevec[r]+mui[r,]*scalevec[r] + si*scalevec[3-r]*((response[[3-r]]-eta[[3-r]])-mui[3-r,]-REyes[3-r]*(result$Z[[3-r]]%*%delta[[3-r]][i-1,])[,,drop=T])

       yhattemp=eta[[r]]+mui[r,]+REyes[r]*(result$Z[[r]]%*%delta[[r]][i-1,])[,,drop=T] + centervec[r]/scalevec[r]
    #    result$yhat[,r]=result$yhat[,r]+rmvnorm(1,yhattemp, diag(cond.vari))
 #       result$yhat[,r]=result$yhat[,r]+yhattemp
        result$int.sample[intnr,r]=mean(yhattemp)
        result$yhat.sample[[r]][,intnr]=yhattemp
        if (r==2) intnr=intnr+1
      }


    #======================
    # Random Effects
      if (REyes[r]){
        b.sig_delta[[r]]= b.sig_delta_0 + 1/2 * sum(delta[[r]][i-1,]^2)   #a.sig_delta is only updated once
        sig_delta2[[r]][i,] =(rgamma(n=1, a.sig_delta[[r]], b.sig_delta[[r]]))^(-1)

        VAR.delta = solve(t(cond.vari^(-1)* result$Z[[r]])%*%result$Z[[r]]+diag(1/sig_delta2[[r]][i,],ncol(result$Z[[r]])))
        mu.delta= VAR.delta%*%((t(cond.vari^(-1)*result$Z[[r]])%*%(response[[r]] - (mui[r,] +eta[[r]] +offset))))
        delta[[r]][i,] = rmvnorm(n=1, mean=mu.delta, sigma=VAR.delta,method=method)
      }



   #======================
    # Parametric effects

      eta_tilde= eta[[r]] - f.gamma[[r]] # ==0 if numSplines_first==0 or numSplines_second==0
      offset_tilde=mui[r,] +eta_tilde +offset +REyes[r]*(result$Z[[r]]%*%delta[[r]][i,])[,,drop=T]   #
      if(ncol(V[[r]])>0){
        VAR.gamma[[r]] = solve(t(cond.vari^(-1)* V[[r]])%*%V[[r]])
        mu.gamma[[r]]= VAR.gamma[[r]]%*%((t(cond.vari^(-1)*V[[r]])%*%(response[[r]] - offset_tilde)))
        gamma[[r]][i,] = rmvnorm(n=1, mean=mu.gamma[[r]], sigma=VAR.gamma[[r]],method=method)

        f.gamma[[r]]= V[[r]] %*% gamma[[r]][i,]
      }
      eta[[r]]= eta_tilde + f.gamma[[r]]  # Updating of the predictor

    #======================
    # Nonparametric effects
      go=F
      if (r==1 & (numSplines_first>0|numSpatial_first>0)){go=T;npitmin=1;npitmax=numSplines_first+numSpatial_first}
      if (r==2 & (numSplines_second>0|numSpatial_second>0)){go=T;npitmin=numSplines_first+numSpatial_first+1;npitmax=length(npvars)}
       if (go)
       for (j in npitmin:npitmax){
        # spatial effect or nonparametric effect?
          if ((numSpatial_first>0 & j== numSplines_first+1) | (numSpatial_second>0 & j== numSplines_first+numSpatial_first+numSplines_second+1))
            {K= K.mrf[[r]]; a.tau= a.tau.sp[[r]]; b.tau_0=b.tau.sp_0}
          else {K= K.rw; a.tau= a.tau.rw; b.tau_0=b.tau.rw_0}

        eta_tilde= eta[[r]] - f[[j]]  #== eta[[1]] - (X_first[[j]] %*% beta_first[[j]][i-1,] -i.constraint_j[i-1])
        offset_tilde=  mui[r,]+ eta_tilde +offset+REyes[r]*(result$Z[[r]]%*%delta[[r]][i,])[,,drop=T]  #

        # Updating of tau2
          #a.tau is only updated once
          b.tau= b.tau_0 + 1/2 * t(beta[[j]]) %*% K %*% beta[[j]]
          tau2 =(rgamma(n=1, a.tau, b.tau))^(-1)
        # Updating of beta
          P= t(cond.vari^(-1)* X[[j]])%*%X[[j]]+ tau2^(-1) * K
          L=chol(P)
          nu= forwardsolve(L,(t(cond.vari^(-1)*X[[j]]) %*% (response[[r]]- offset_tilde)), upper.tri = TRUE, transpose = TRUE)
          mu = backsolve(L,nu)
          # beta[i-1] -> beta[i]
            beta[[j]]= mu + solve(L,rnorm(dim(X[[j]])[2],0,1))

        f[[j]]= X[[j]] %*% beta[[j]]

        # centering (varying coefficients are not centered)
          if (byvars[j]=="NA"){  #no varying coefficient
          i.constraint_j= mean(f[[j]])
            f[[j]]= f[[j]] -i.constraint_j
          }
        # Updating of eta[[r]]
          eta[[r]]= eta_tilde + f[[j]] #== eta[[r]]= eta[[r]] - (X[[j]] %*%beta[[j]][i-1,] - i.constraint_j[i-1]) + X[[j]] %*% beta[[j]][i,] - i.constraint_j[i]

        # Record Samples
          if (i>(numBurnIn+1) & (i-(numBurnIn+1))%%thin==0){
            if (byvars[j]=="NA"){
              f.sum[[j]]=f.sum[[j]]+f[[j]]
              f.rec[[j]][,fnr[[j]]]=f[[j]]
              if (!is.null(x.pred[[j]])){
#                f.pred[[j]]=f.pred[[j]]+ X.pred[[j]] %*% beta[[j]] - i.constraint_j
                f.pred[[j]]=X.pred[[j]] %*% beta[[j]] - i.constraint_j
                f.pred.sum[[j]]=f.pred.sum[[j]]+f.pred[[j]]
                f.pred.rec[[j]][,fnr[[j]]]=f.pred[[j]]
              }
            }
            if (record.npcoef| byvars[j]!="NA") beta.rec[[j]][fnr[[j]],]=beta[[j]]
            fnr[[j]]=fnr[[j]]+1
          }
      }                                                                                                                                                                                                                                        # -(mui[3-r,])
  }


  #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  # Unstandardized deviance
    Dev[i]=NA#log(2*pi) +crossprod(response[[1]]-eta[[1]])  +  log(2*pi*sigma2_1[i])+sigma2_1[i]^(-1)*crossprod(sigma12[i]*(response[[1]][z_aux==1]-eta[[1]][z_aux==1])-(response[[2]]-eta[[2]]))

  if (i==51)  {cat("\nEstimated time to end: ", ((proc.time()[3]-p8[3])*(numBurnIn+numSamples)/50)%/%60,"min",round(((proc.time()[3]-p8[3])*(numBurnIn+numSamples)/50)%%60,0),"sec\n");cat("\nProgress:\n "); if (progressBar) pb = txtProgressBar(style=3)}
}
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
# end of loop

#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
# Calculation of final estimators

  #*********************
  # Covariance components
    if (density.method=="conley"){
      result$Istar.sample = Istardraw[seq((numBurnIn+2),(numBurnIn+numSamples+1),thin)]
      result$alpha.sample=  alphadraw[seq((numBurnIn+2),(numBurnIn+numSamples+1),thin)]
      result$nmix=list(probdraw=matrix(c(rep(1,length(thetaNp1draw))),ncol=1),zdraw=NULL,compdraw=thetaNp1draw)
      attributes(result$nmix)$class=c("bayesIV.nmix")
    }
    else{
      result$density$mui1=result$density$mui1    /(numSamples%/%thin)
      result$density$mui2=result$density$mui2    /(numSamples%/%thin)
      result$density$sigma1=result$density$sigma1  /(numSamples%/%thin)
      result$density$sigma2=result$density$sigma2 /(numSamples%/%thin)
      result$density$sigma12=result$density$sigma12 /(numSamples%/%thin)

      result$density$fun1=result$density$fun1/(numSamples%/%thin)
      result$density$fun2=result$density$fun2/(numSamples%/%thin)
      result$density$f=result$density$f/(numSamples%/%thin)
      result$density$varnames=c("eps1","eps2")
    }


int=numeric()
for (r in 1:2){
  result$yhat.sample[[r]]=result$yhat.sample[[r]] *scalevec[r]
  result$yhat[,r]= rowMeans( result$yhat.sample[[r]])
}


result$int.sample=t(t(result$int.sample)*scalevec)
result$intercepts=colMeans(result$int.sample)-sapply(V.unscaled, function(x) sum(colMeans(x)))
result$ci.intercepts=apply(result$int.sample, MARGIN=2, quantile, probs= c(0.025, 0.5, 0.975))

 # for (r in 1:2){
 #  result$cmean[[r]]=result$cmean[[r]]/(numSamples%/%thin)
 ##  result$eps[[r]]=result$eps[[r]]/(numSamples%/%thin)
  # result$cmeanB[[r]]=result$cmeanB[[r]]/(numSamples%/%thin)
 #  result$eps2[[r]]=result$eps2[[r]]/(numSamples%/%thin)
 # }


  #*********************
  # Parametric effects
    # Discarding burn-in, thinning and rescaling
      gamma_first.sample = as.data.frame(gamma[[1]][seq((numBurnIn+2),(numBurnIn+numSamples+1),thin), ,drop=F])*scale1
      gamma_second.sample = as.data.frame(gamma[[2]][seq((numBurnIn+2),(numBurnIn+numSamples+1),thin), ,drop=F])*scale2
gamma_first.sample[,1] =gamma_first.sample[,1]+centervec[1]#- sum(colMeans(V.unscaled[[1]][,-1,drop=F]))
gamma_second.sample[,1] =gamma_second.sample[,1]+centervec[2]-(!is.null(as.data.frame(V[[2]])[[response.first]]))*centervec[1]#- sum(colMeans(V.unscaled[[2]][,-1,drop=F]))
      result$coefficients.sample=list(first=gamma_first.sample,second =gamma_second.sample)
      if (ncol(V[[1]])>0) dimnames(gamma_first.sample)[[2]]= dimnames(result$coefficients.sample[[1]])[[2]]= g_first
      if (ncol(V[[2]])>0) dimnames(gamma_second.sample)[[2]]=dimnames(result$coefficients.sample[[2]])[[2]]= g_second

      if (!is.null(gamma_second.sample[[response.first]])) gamma_second.sample[[response.first]]=result$coefficients.sample[[2]][[response.first]]=gamma_second.sample[[response.first]]/scale1


      gamma_first.mean   = colMeans(gamma_first.sample)
      gamma_second.mean   = colMeans(gamma_second.sample)
      result$coefficients=list(first=gamma_first.mean,second=gamma_second.mean)

      sd.first=apply(gamma_first.sample,2, function(x) sqrt(var(x)))
      sd.second=apply(gamma_second.sample,2, function(x) sqrt(var(x)))
      result$sd.coefficients=list(first=sd.first,second=sd.second)

    # Quantiles
      ci_gamma.first = apply(gamma_first.sample, MARGIN=2, quantile, probs= c(0.025, 0.5, 0.975))
      ci_gamma.second = apply(gamma_second.sample, MARGIN=2, quantile, probs= c(0.025, 0.5, 0.975))
      result$ci.coefficients=list(first=ci_gamma.first,second=ci_gamma.second)

  # random effects
    result$sigma.random.sample= result$sigma.random= result$coefficients$random=result$coefficients.sample$random=list()
    scalevec=c(scale1,scale2)
    for (r in 1:2)
      if (REyes[r]){
        result$sigma.random.sample[[r]] = as.data.frame(sqrt(sig_delta2[[r]][seq((numBurnIn+2),(numBurnIn+numSamples+1),thin), ,drop=F]))*scalevec[r]
        result$sigma.random[[r]]  = mean(result$sigma.random.sample[[r]])
        result$coefficients.sample$random[[r]] = as.data.frame(delta[[r]][seq((numBurnIn+2),(numBurnIn+numSamples+1),thin), ,drop=F])*scalevec[r]
        result$coefficients$random[[r]]  = colMeans(result$coefficients.sample$random[[r]])
    }


  if (SCALE){
    if (length(npvars)>0) npcovariates=npcovariates.unscaled
    V=V.unscaled
  }

  #*********************
  # Nonparametric effects
    ci.f.pred= result$ci.f= result$f.by= result$f=result$ci.npcoef=result$npcoef=  list()
    if (length(npvars)>0){
    result$eq=numeric()
    # rescaling

      if (numSplines_first>0|numSpatial_first>0){
        result$eq=rep(1,(numSplines_first+numSpatial_first))
        for (i in 1:(numSplines_first+numSpatial_first)){
          f.sum[[i]] =f.sum[[i]] *scale1
          f.rec[[i]]=f.rec[[i]]  *scale1
          if (length(beta.rec[[i]])>1) beta.rec[[i]]=beta.rec[[i]]*scale1
          if (!is.null(x.pred[[i]])){
            f.pred.sum[[i]]=f.pred.sum[[i]]*scale1
            f.pred.rec[[i]]=f.pred.rec[[i]]*scale1
          }
        }
      }
      if (numSplines_second>0|numSpatial_second>0){
        result$eq=c(result$eq,rep(2,numSplines_second+numSpatial_second))
        for (i in (numSplines_first+numSpatial_first+1):length(npvars)){
          f.sum[[i]] =f.sum[[i]] *scale2
          f.rec[[i]]=f.rec[[i]]  *scale2
          if (length(beta.rec[[i]])>1) beta.rec[[i]]=beta.rec[[i]]*scale2
          if (!is.null(x.pred[[i]])){
            f.pred.sum[[i]]=f.pred.sum[[i]]*scale2
            f.pred.rec[[i]]=f.pred.rec[[i]]*scale2
          }
        }
      }
      #get fitted values
      for (j in 1:length(npvars)){
        if (length(beta.rec[[j]])>1){
          result$npcoef[[j]]   = colMeans(beta.rec[[j]])
          result$ci.npcoef[[j]]= apply(beta.rec[[j]], MARGIN=2, quantile, probs= c(0.025, 0.10, 0.90, 0.975))
        }
        else result$npcoef[[j]] = result$ci.npcoef[[j]]= 0

        if (length(f.rec[[j]])>1){
          result$f[[j]]= f.sum[[j]] /(numSamples%/%thin)
          dim(f.rec[[j]])
          result$ci.f[[j]]= t(apply(f.rec[[j]], MARGIN=1, quantile, probs= c(0.025, 0.10, 0.90, 0.975)))
         result$ci.f.uncentered[[j]]=t(apply( matrix(mapply(FUN=function(f,int) f+int, t(f.rec[[j]]),result$int.sample[,result$eq[j]],SIMPLIFY =T),ncol=nrow(f.rec[[j]])), MARGIN=2, quantile, probs= c(0.025, 0.10, 0.90, 0.975)))
     if (!is.null(x.pred[[j]])){
            f.pred[[j]]=f.pred.sum[[j]] /(numSamples%/%thin)
            ci.f.pred[[j]]= t(apply(f.pred.rec[[j]], MARGIN=1, quantile, probs= c(0.025, 0.10, 0.90, 0.975)))
          }
        }
        else result$f[[j]] = result$ci.f[[j]]= f.pred[[j]]=0

        if (byvars[j]!="NA"){
          result$f[[j]]=X[[j]]%*%result$npcoef[[j]]
          result$f.by[[j]]=Xby[[j]]%*%result$npcoef[[j]]
          temp.sample=apply(beta.rec[[j]], 1, function(x) Xby[[j]]%*%x)
          result$ci.f[[j]]= t(apply(temp.sample, MARGIN=1, quantile, probs= c(0.025, 0.10, 0.90, 0.975)))
          rm(temp.sample)
        }
        else result$f.by[[j]]=0
      }
      names(result$npcoef) = names(result$f) = names(result$f.by) = names(result$ci.npcoef)= names(result$ci.f)  = npvars
      if (!all(unlist(lapply(x.pred,is.null)))){
        names(f.pred)=npvars[byvars=="NA"]
        for (j in 1:length(npvars)) if (byvars[j]!="NA") names(f.pred)= paste(byvars[j],"*",npvars[j],sep="") 
      }
      for (j in 1:length(npvars)) if (byvars[j]!="NA") names(result$f)[j]=paste(byvars[j],"*",npvars[j],sep="")
    }

#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
# "CI's" for spatial effects
  if (numSpatial_first>0) {
    pcat80.first=mapply(FUN=function(x,y) {if (x<0 & y<0) return(-1)
                                         if ((x<=0 & y>=0)|(x>=0 & y<=0)) return(0)
                                         if (x>0 & y>0) return(1)},
                      t(result$ci.f[[numSplines_first+1]][,2]), t(result$ci.f[[numSplines_first+1]][,3]))
    pcat95.first=mapply(FUN=function(x,y) {if (x<0 & y<0) return(-1)
                                         if ((x<=0 & y>=0)|(x>=0 & y<=0)) return(0)
                                         if (x>0 & y>0) return(1)},
                      t(result$ci.f[[numSplines_first+1]][,1]), t(result$ci.f[[numSplines_first+1]][,4]))
  }
  else pcat80.first=pcat95.first=NULL
  if (numSpatial_second>0) {
    pcat80.second=mapply(FUN=function(x,y) {if (x<0 & y<0) return(-1)
                                         if ((x<=0 & y>=0)|(x>=0 & y<=0)) return(0)
                                         if (x>0 & y>0) return(1)},
                      result$ci.f[[numSplines_first+numSpatial_first+numSplines_second+1]][,2],result$ci.f[[numSplines_first+numSpatial_first+numSplines_second+1]][,3])
    pcat95.second=mapply(FUN=function(x,y) {if (x<0 & y<0) return(-1)
                                         if ((x<=0 & y>=0)|(x>=0 & y<=0)) return(0)
                                         if (x>0 & y>0) return(1)},
                      result$ci.f[[numSplines_first+numSpatial_first+numSplines_second+1]][,1],result$ci.f[[numSplines_first+numSpatial_first+numSplines_second+1]][,4])
  }
  else pcat80.second=pcat95.second=NULL

  result$pcat80=list(first=pcat80.first,second=pcat80.second)
  result$pcat95=list(first=pcat95.first,second=pcat95.second)

#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
# DIC
  # mean of the latent variable in first equation
#    z.mean = z.sum / (numSamples%/%thin)
#  Dev.sample = Dev[seq((numBurnIn+2), (numBurnIn+numSamples+1), thin)]
#  Dev.mean = mean(Dev.sample)
  Dev.post = 0#log(2*pi) + crossprod(z.mean - fitted.first)   +   log(2*pi*result$sigma2_1) + result$sigma2_1^(-1) * crossprod(result$sigma12*(z.mean[z_aux==1]-fitted.first[z_aux==1]) - (response[[2]]-fitted.second))
  result$DIC = NA#2*Dev.mean - Dev.post


#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
# Posterior probabilities for parametric effects
  pl0.first= apply(gamma_first.sample,2,function(x) 1/length(x)*sum(x<0))
  pu0.first= apply(gamma_first.sample,2,function(x) 1/length(x)*sum(x>0))
  pl0.second= apply(gamma_second.sample,2,function(x) 1/length(x)*sum(x<0))
  pu0.second= apply(gamma_second.sample,2,function(x) 1/length(x)*sum(x>0))

  result$pvalues.onesided=list(first=data.frame(cbind(pl0.first,pu0.first),row.names=g_first),second=data.frame(cbind(pl0.second,pu0.second),row.names=g_second))
  result$pvalues.twosided=list(first=apply(result$pvalues.onesided$first,1,function(x) 2*min(x)),second=apply(result$pvalues.onesided$second,1,function(x) 2*min(x)))

p9= proc.time()
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


  result$npcoef.sample=beta.rec
  if (return.f) {result$f.sample= f.rec;result$f.pred.sample= f.pred.rec}
  if (!all(unlist(lapply(x.pred,is.null)))){
   result$f.pred=f.pred
   result$ci.f.pred=ci.f.pred
  }
  result$responses=list(response[[1]],response[[2]])
  names(result$responses)=c(response.first,response.second)

  result$covariates=list(parametric=V,nonparametric=npcovariates,by=byvars.list)
  result$x.pred=x.pred
  result$formulas=list(first=first,second=second)
  result$numSplines=list(first=numSplines_first,otucome=numSplines_second)
  result$numSpatial=list(first=numSpatial_first,second=numSpatial_second)
  result$sp=list(first=sp_first,second=sp_second)
  result$numBurnIn=numBurnIn
  result$numSamples=numSamples
  result$thin=thin
  result$density$scale1=scale1
  result$density$scale2=scale2
  result$density$mu1=mu1
  result$density$mu2=mu2


#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
# OUTPUT
if (progressBar) close(pb)
cat("\n\nTotal time elapsed\n");print(p9-p0)
class(result)=c("bayesIV","bayesIV",density.method)
summary.bayesIV(result)

#scbs
  if (length(result$covariates$nonparametric)>0 &scbs) result$scb=plot.bayesIV(result,scb=TRUE,scb.level=scb.level,PLOT=F,scb.method="TK",center=scb.center)          #  scb.method=scb.method

# Plot sampling paths of parametric effects
  if (plotpaths & (ncol(V[[1]])>0 | ncol(V[[2]])>0))
    plotPaths(cbind(gamma[[1]], gamma[[2]]), names=c(g_first,g_second),numBurnIn=numBurnIn)

if (!return.samples) result$coefficients.sample=result$npcoef.sample= NULL
invisible(result)
}
