
##############################################################
#                                                            #
# Requires packages mvtnorm and splines. Package BayesX 0.2  #
# is required if spatial effects are included.               #
#                                                            #
##############################################################
bayesIVgauss =
function(first, second, random=list(NULL,NULL), data=parent.frame(),numBurnIn=2000, numSamples=20000, thin=20
, contrasts.first =NULL, contrasts.second =NULL
, graph= NULL
, x.pred=NULL
, numKnots=40, knotchoice="equidistant", degree=3, rw.order=2
, SCALE=T
, prior.cov=list(s.Sigma=3,S.Sigma=.2*diag(2))
, prior.random= c(0.001,0.001)
,  prior.tau=c(0.001,0.001), prior.tau.sp=c(0.001,0.001)
, seed=NULL
, scbs=F,scb.level=0.95,scb.method="TK"
, record.npcoef=F, return.samples=T, return.f=T
, mgcv=F, plotpaths=F#, plotpaths.style="separate"
, progressBar=T
, method="chol"){

prior.cov$nu=prior.cov$s.Sigma
prior.cov$V=prior.cov$S.Sigma

###############################################

result=list()
date=format(Sys.time(), "%Y-%m-%d--%Hh%Mm%Ss")
result$date=date

p0=proc.time()
################################################
# Read formulas

  #------------------
  #first equation
    # Interpret formula of first equation
      np=interpret.gam(first)
    # recognize on which location in the formula there is a s() (for smooth functions) or sp() (for spatial effects)
      tf.first = terms.formula(first, specials = c("s", "sp"))
      sp_first = attr(tf.first, "specials")$sp
    # Number of nonparametric effects
      numSplines_first = length(np$smooth.spec)
      numSpatial_first = length(sp_first)
    # get smooth parameters and 'by' variables
      npvars.first= byvars.first= numeric(numSplines_first)
      if (numSplines_first>0){
        for (j in 1:(numSplines_first)){
          npvars.first[j]=np$smooth.spec[[j]]$term
          byvars.first[j]=np$smooth.spec[[j]]$by
        }
      }
########
 #WIRD PROBLEME MIT BYVARS VERURSACHEN
  #     spvar.first=all.vars(first, unique = FALSE)[-which(all.vars(first, unique = FALSE) %in% byvars.first[byvars.first!="NA"])][sp_first]
    spvar.first=all.vars(first, unique = FALSE)[sp_first]

  #--------------------
  # second equation
    # interpret formula of second equation
      np=interpret.gam(second)
    # recognize on which location in the formula there is a s() (for smooth functions) or sp() (for spatial effects)
      tf.second = terms.formula(second, specials = c("s", "sp"))
      sp_second = attr(tf.second, "specials")$sp
    # Number of nonparametric effects
      numSplines_second = length(np$smooth.spec)
      numSpatial_second = length(sp_second)
    # get smooth parameters and by variables
      npvars.second= byvars.second= numeric(numSplines_second)
      if (numSplines_second>0){
        for (j in 1:(numSplines_second)){
          npvars.second[j]=np$smooth.spec[[j]]$term
          byvars.second[j]=np$smooth.spec[[j]]$by
        }
      }
    # spatial effects
 ########
 #WIRD PROBLEME MIT BYVARS VERURSACHEN
 #     spvar.second=all.vars(second, unique = FALSE)[-which(all.vars(second, unique = FALSE) %in% byvars.second[byvars.second!="NA"])][sp_second]
      spvar.second=all.vars(second, unique = FALSE)[sp_second]

  #------------------
  # Combined
    # Auxillary variables -> for spatial effects no varying coefficients are implemented, these variables are only needed as wild-cards in 'byvars'
      if (numSpatial_first>0) byvarSp.first_aux="NA"  else byvarSp.first_aux= numeric(0)
      if (numSpatial_second>0) byvarSp.second_aux="NA"  else byvarSp.second_aux= numeric(0)
    # Vector of nonparametric effects including spatial effects if included  (note the order of first&second&spatial!)
      npvars=c(npvars.first,spvar.first,npvars.second,spvar.second)
      byvars=c(byvars.first, byvarSp.first_aux, byvars.second, byvarSp.second_aux)


################################################
# Spatial Data
  # Estimate spatial effects?
    geo=FALSE
    if ((is.null(graph)==F) & (numSpatial_second>0 | numSpatial_first>0)) geo=TRUE
  # Read spatial data
    if (geo){
      pmat=K.mrf=regions=numRegions=list()
      if (numSpatial_first>0){
        pmat[[1]] =pen.mrf(graph)
        regions[[1]]=pmat[[1]]$regions
        numRegions[[1]]=length(regions[[1]])
        # penalty matrix
          K.mrf[[1]]=pmat[[1]]$K.mrf
      }
      if (numSpatial_second>0){
        pmat[[2]] =pen.mrf(graph)
        regions[[2]]=pmat[[2]]$regions
        numRegions[[2]]=length(regions[[2]])
        # penalty matrix
          K.mrf[[2]]=pmat[[2]]$K.mrf
      }
    }
################################################
# Model matrices and responses
  #=============================================================================
  # Responses and model matrices of parametric covariates
    # Model formula for the strictly parametric part of the model.
      firstPar= read.formula(first)$pf
      secondPar= read.formula(second)$pf
    # Read formulas and convert to model frames
      frame.firstPar= model.frame(firstPar,data=data)
      frame.secondPar= model.frame(secondPar, data=data)
    # Responses
      response = list()
      response[[1]] = model.response(frame.firstPar)
      response[[2]] = model.response(frame.secondPar)#[response[[1]] ==1]
      response.first=all.vars(first)[1]
      response.second=all.vars(second)[1]
    # Design matrices of parametric effects
      V = VV= list()
      V[[1]] = model.matrix(firstPar, frame.firstPar,contrasts.arg=contrasts.first)
      V[[2]] = model.matrix(secondPar, frame.secondPar,contrasts.arg=contrasts.second)#[response[[1]]==1, ,drop=F]
    # Name vector of parametric effects including Intercept if included
      g_first=dimnames(V[[1]])[[2]]
      g_second=dimnames(V[[2]])[[2]]

  # Design matrices and initialisation for random effects
    REyes=c(!is.null(random[[1]]),!is.null(random[[2]]))
    result$Z=ZZ=a.sig_delta=b.sig_delta=sig_delta2=delta=list()
    for (r in 1:2){
      if (REyes[r]){
        m=length(unique(random[[r]]))
        result$Z[[r]]=matrix(0,ncol=m,nrow=length(response[[r]]))
        for (i in 1:m) result$Z[[r]][(random[[r]]==unique(random[[r]])[i]),i]=1
      ZZ[[r]]=crossprod(result$Z[[r]])
      a.sig_delta_0 =prior.random[1]
      b.sig_delta_0 =prior.random[2]
      a.sig_delta[[r]]=a.sig_delta_0+m/2
      delta[[r]] = matrix(data=0, nrow=(numBurnIn+numSamples+1), ncol=m)   #dim(V[[2]])[2]=length(gamma[[2]])
      }else{ result$Z[[r]]=0;delta[[r]] =  matrix(data=0, nrow=(numBurnIn+numSamples+1), ncol=1)  }  #  delta[[r]] =
      sig_delta2[[r]] = matrix(data=0, nrow=(numBurnIn+numSamples+1), ncol=1)
    }

    # Scaling
    if (SCALE){
      V.unscaled=V
      scale1=sqrt(var(response[[1]]))
      scale2=sqrt(var(response[[2]]))
      mu1=mean(response[[1]])
      mu2=mean(response[[2]])
        response[[1]] =response[[1]]/scale1# scale(response[[1]],scale=T,center=F)
        response[[2]] =response[[2]]/scale2# scale(response[[2]],scale=T,center=F)

      # only center not factor variables and exclude intercept
        fac=logical()
        for (i in 1:length(attr(terms(firstPar),"term.labels"))){
         fac[i]=try((attr(V[[1]],"contrasts")[[attr(terms(firstPar),"term.labels")[i]]]=="contr.treatment") )
        }
        fac[is.na(fac)]=FALSE
        fac=c(T,fac)
        if (!is.null(dim(V[[1]][,!fac]))) V[[1]][,!fac]= (apply(V[[1]][,!fac],2,function(x) x-mean(x)))

        fac=logical()
        for (i in 1:length(attr(terms(secondPar),"term.labels")))fac[i]=(attr(V[[2]],"contrasts")[[attr(terms(secondPar),"term.labels")[i]]]=="contr.treatment")
        fac[is.na(fac)]=FALSE
        fac=c(T,fac)
      if (!is.null(dim(V[[2]][,!fac]))) V[[2]][,!fac]= (apply(V[[2]][,!fac],2,function(x) x-mean(x)))
      
      if (!is.null(as.data.frame(V[[2]])[[response.first]])) V[[2]][,response.first] =response[[1]]
    } else {scale1=scale2=1;mu1=mu2=0}
    scalevec=c(scale1,scale2)

#      if (SCALE){
#        V.unscaled=V
#        scale1=sqrt(var(response[[1]]))
#        scale2=sqrt(var(response[[2]]))
#        response[[1]] =response[[1]]/scale1# scale(response[[1]],scale=T,center=F)
#        response[[2]] =response[[2]]/scale2# scale(response[[2]],scale=T,center=F)
#        if (!is.null(as.data.frame(V[[2]])[[response.first]])) V[[2]][,response.first] =response[[1]]
#      } else scale1=scale2=1

    # Auxillary matrices
      VV[[1]]= crossprod(V[[1]])
      VV[[2]]= crossprod(V[[2]])

  #=============================================================================
  # List of the nonparametric covariates
    npcovariates = lapply(as.list(npvars), function(x) data[[x]])
    byvars.list  = lapply(as.list(byvars), function(x) data[[x]])      # List of variables given by a 'by=' argument for varying coefficients
    if (length(npvars)>0){
      names(npcovariates)=npvars
      for (j in 1:length(npvars)) if (byvars[j]!="NA") names(byvars.list)[j] = byvars[j]
      npcovariates.unscaled=npcovariates
      if (SCALE){
######
##
##was ist mit byvars?
##
####
        if (!is.null(npcovariates[[response.first]])){whichendo=which(npvars==response.first); npcovariates[[response.first]] =response[[1]]}else whichendo=NULL
      }else whichendo=NULL
    }
  #-----------------------------------------------------------------------------
  # Model matrices of the nonparametric covariates
    X= XX= Xby= X.pred=f.pred.sum= list()
    if (length(npvars) > 0){
      if (numSpatial_first==1) npcovariates[[spvar.first]]=as.numeric(as.character(npcovariates[[spvar.first]]))
      if (numSpatial_second==1) npcovariates[[spvar.second]]=as.numeric(as.character(npcovariates[[spvar.second]]))
      # Prediction on a given grid if specified
        if (is.null(x.pred)) {
          x.pred=list()
          length(x.pred)=length(npvars)
        }
        else 
        if (!is.list(x.pred)){
            temp=list()
            for (i in 1:length(npvars)) temp[[i]]=x.pred
            x.pred=temp
            rm(temp)
        }
      for (j in 1:length(npvars)){    #first first, then  second equation
        # Write design matrix of spatial covariate
          if (numSpatial_first>0 & j==numSplines_first+1){
            X[[j]] = matrix(0,length(npcovariates[[j]]),numRegions[[1]])
              for(i in 1:numRegions[[1]])
                X[[j]][which(npcovariates[[j]]==regions[[1]][i]),i] = 1     # 1 if y_i1 was observed in region s
            if (!is.null(x.pred[[j]])){
              X.pred[[j]] = matrix(0,length(x.pred[[j]]),numRegions[[1]])
              f.pred.sum[[j]]=0
                for(i in 1:numRegions[[1]])
                  X.pred[[j]][which(x.pred[[j]]==regions[[1]][i]),i] = 1
            }
          }
          if (numSpatial_second>0 &  j==numSplines_first+numSpatial_first+numSplines_second+1){
            X[[j]] = matrix(0,length(npcovariates[[j]]),numRegions[[2]])
              for(i in 1:numRegions[[2]])
                X[[j]][which(npcovariates[[j]]==regions[[2]][i]),i] = 1      # 1 if y_i2 was observed in region s
            if (!is.null(x.pred[[j]])){        
              X.pred[[j]] = matrix(0,length(x.pred[[j]]),numRegions[[2]])
              f.pred.sum[[j]]=0
                for(i in 1:numRegions[[2]])
                  X.pred[[j]][which(x.pred[[j]]==regions[[2]][i]),i] = 1
            }              
          }
        # write B-spline basis design matrix of nonparametric effects into list element j
          if (j<=numSplines_first | (j>numSplines_first+numSpatial_first&j<=numSplines_first+numSpatial_first+numSplines_second)){
            X[[j]]=bspline(npcovariates[[j]],numKnots=numKnots,knotchoice=knotchoice,degree=degree)
            if (!is.null(x.pred[[j]])){
              X.pred[[j]]=bspline(x.pred[[j]],numKnots=numKnots,knotchoice=knotchoice,degree=degree)
              f.pred.sum[[j]]=0
            }
          }
        if (byvars[j]!="NA"){
          Xby[[j]]=X[[j]]
          X[[j]]= diag(byvars.list[[j]]) %*% X[[j]]
        }
       XX[[j]]= crossprod(X[[j]])
      }
    }

#%%%%%%%%%%%%%%%
# discard unnecessary matrices
  rm(frame.firstPar, frame.secondPar)
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

################################################
# Some error search:
  # Intercept: Either 0, indicating no intercept is to be fit, or 1 indicating that an intercept is to be fit
    int_first = attr(tf.first,"intercept")
    int_second = attr(tf.second,"intercept")

  if ((int_first==0 & (numSplines_first>0|numSpatial_first>0))|(int_second==0 & (numSplines_second>0|numSpatial_second>0))) stop("When nonparametric effects are included, an intercept is required.")
  if ((numSpatial_first>0|numSpatial_second>0) && !geo) stop("Graph not correctly specified")
  if (scbs==T & return.f==F){warning("return.f set to TRUE to allow for SCBs");return.f=T}



##############################################################
# Penalty matrix for smooth functions
  D= diff(diag(numKnots+2),differences=rw.order)
  K.rw=crossprod(D)

##############################################################
# Initialization
  if (!is.null(seed)) set.seed(seed)
  cond.var= VAR.gamma= mu.gamma= gamma= f.gamma= eta= list()
  sigma11= sigma12= sigma22= Dev =numeric(numBurnIn+numSamples+1)
  gamma[[1]] = matrix(data=0, nrow=(numBurnIn+numSamples+1), ncol=dim(V[[1]])[2])   #dim(V[[1]])[2]=length(gamma[[1]])
  gamma[[2]] = matrix(data=0, nrow=(numBurnIn+numSamples+1), ncol=dim(V[[2]])[2])   #dim(V[[2]])[2]=length(gamma[[2]])

  #***************************************************************************
  # Inizialize betas, f's etc.
    beta= beta.rec= f= f.sum=f.rec= fnr= f.pred=f.pred.rec=list()
    if (length(npvars)>0){
      f[1:length(npvars)]= f.sum[1:length(npvars)]= beta[1:length(npvars)]= 0
      if (!all(unlist(lapply(x.pred,is.null)))) f.pred[1:length(npvars)]= 0
      fnr[1:length(npvars)]=1
      for (j in 1:length(npvars)) {
        beta[[j]]= numeric(dim(X[[j]])[2])    #betas have a starting value zero
        if (record.npcoef| byvars[j]!="NA")     #beta is only recorded if desired
          beta.rec[[j]]=matrix(data=0, ncol=dim(X[[j]])[2],nrow=floor(numSamples/thin))
        else beta.rec[[j]]=numeric()
        if (byvars[j]=="NA"){  
          f.rec[[j]]=matrix(data=0, nrow=dim(X[[j]])[1],ncol=floor(numSamples/thin))
          if (!is.null(x.pred[[j]])) f.pred.rec[[j]]=matrix(data=0, nrow=dim(X.pred[[j]])[1],ncol=floor(numSamples/thin))
         }
        else f.rec[[j]]=numeric()
      }
      names(beta.rec)=names(f.rec)=npvars
    }


################################################
# Comperative estimation
  #------------------
  # Auxillary formulas
    # Formulas with all covariates parametric (including those nonparametrically defined)
      fakeformula.first=as.formula(paste(response.first,"~",paste(c(all.vars(firstPar)[-1],npvars.first), collapse="+")))
      fakeformula.second=as.formula(paste(response.second,"~",paste(c(all.vars(secondPar)[-1],npvars.second), collapse="+")))
    # Formulas without spatial effects
      covariates.first=as.vector(attr(tf.first,"term.labels"))
        if (numSpatial_first>0) covariates.first=covariates.first[-(sp_first-1)]
      covariates.second=as.vector(attr(tf.second,"term.labels"))
        if (numSpatial_second>0) covariates.second=covariates.second[-(sp_second-1)]

      mgcvformula.first=as.formula(paste(response.first,"~",paste(covariates.first, collapse="+")))
      mgcvformula.second=as.formula(paste(response.second,"~",paste(covariates.second, collapse="+")))

  #------------------
  # Univariate estimation with mgcv package
    result$f.mgcv=result$f.mgcv.pred=list()
    if (mgcv){
      #cat("\nUnivariate estimation using package mgcv\n")
      if (is.data.frame(data)) datam=data else datam=NULL
      mgcv.second=gam(mgcvformula.second, data=datam)
      mgcv.first=gam(mgcvformula.first, data=datam)
      rm(datam)
      #print(summary(mgcv.first))
      #print(summary(mgcv.second))

      coef.mgcv_first=as.vector(mgcv.first$coefficients[1:ncol(V[[1]])])
      coef.mgcv_second=as.vector(mgcv.second$coefficients[1:ncol(V[[2]])])
      sd.coef.mgcv_first=sqrt(diag(vcov(mgcv.first))[1:ncol(V[[1]])])
      sd.coef.mgcv_second=sqrt(diag(vcov(mgcv.second))[1:ncol(V[[2]])])

      # values of f 
        f.mgcv.first=predict(mgcv.first,type="terms")
        if (length(attr(mgcv.first$pterms,"term.labels"))>0) f.mgcv.first=f.mgcv.first[,-(1:length(attr(mgcv.first$pterms,"term.labels"))) ,drop=F]
        f.mgcv.second=predict(mgcv.second,type="terms")
        if (length(attr(mgcv.second$pterms,"term.labels"))>0) f.mgcv.second=f.mgcv.second[,-(1:length(attr(mgcv.second$pterms,"term.labels"))) ,drop=F]
        # as list
          if (numSplines_first>0) for (j in 1:ncol(f.mgcv.first)) result$f.mgcv[[j]]=f.mgcv.first[,j]
          if (numSplines_second>0) for (j in 1:ncol(f.mgcv.second)) result$f.mgcv[[j+ncol(f.mgcv.first)]]=f.mgcv.second[,j]
        # Prediction on fixed grid
        if ((numSplines_first>0 | numSplines_second>0) & !all(unlist(lapply(x.pred,is.null)))){
          newdata=V[[1]][1:length(x.pred[[1]]),]
          for (j in 1:numSplines_first) newdata=as.data.frame(cbind(newdata, x.pred[[j]]))
          dimnames(newdata)[[2]]=c(g_first,npvars.first[1:numSplines_first])
          pred.mgcv.first=predict(mgcv.first,newdata=newdata,type="terms")
          if (length(attr(mgcv.first$pterms,"term.labels"))>0) pred.mgcv.first=pred.mgcv.first[,-(1:length(attr(mgcv.first$pterms,"term.labels"))) ,drop=F]

          newdata=V[[2]][1:length(x.pred[[numSplines_first+numSpatial_first+1]]),]
          for (j in 1:numSplines_second) newdata=as.data.frame(cbind(newdata, x.pred[[numSplines_first+numSpatial_first+j]]))
          dimnames(newdata)[[2]]=c(g_second,npvars.second[1:numSplines_second])
          pred.mgcv.second=predict(mgcv.second,newdata=newdata,type="terms")
          if (length(attr(mgcv.second$pterms,"term.labels"))>0) pred.mgcv.second=pred.mgcv.second[,-(1:length(attr(mgcv.second$pterms,"term.labels"))) ,drop=F]

          # as list
            if (numSplines_first>0) for (j in 1:ncol(pred.mgcv.first)) result$f.mgcv.pred[[j]]=pred.mgcv.first[,j]
            if (numSplines_second>0) for (j in 1:ncol(pred.mgcv.second)) result$f.mgcv.pred[[j+ncol(pred.mgcv.first)]]=pred.mgcv.second[,j]
        }
      
      rm(mgcv.first,mgcv.second,f.mgcv.first,f.mgcv.second)
    }
    else{
      coef.mgcv_first=rep(NA,ncol(V[[1]]))
      coef.mgcv_second=rep(NA,ncol(V[[2]]))
      sd.coef.mgcv_first=sd.coef.mgcv_second=NA
    }
    result$coefficients.mgcv=list(first=coef.mgcv_first, second=coef.mgcv_second)
    result$sd.coefficients.mgcv=list(first=sd.coef.mgcv_first,second=sd.coef.mgcv_second)


##############################################################
# Starting values and priors
  # Parametric effects
    if (mgcv){
      gamma[[1]][1,]=coef.mgcv_first
      gamma[[2]][1,]=coef.mgcv_second
      result$coefficients.lm_first=(lm(fakeformula.first,data=data)$coef)[1:ncol(V[[1]])]
      result$coefficients.lm_second=(lm(fakeformula.second,data=data)$coef)[1:ncol(V[[2]])]
    }
    else{
      result$coefficients.lm_first=gamma[[1]][1,]=(lm(fakeformula.first,data=data)$coef)[1:ncol(V[[1]])]
      result$coefficients.lm_second=gamma[[2]][1,]=(lm(fakeformula.second,data=data)$coef)[1:ncol(V[[2]])]
    }
    gamma[[1]][1,] = gamma[[1]][1,]/scale1
    gamma[[2]][1,] = gamma[[2]][1,]/scale2
    f.gamma[[1]]= V[[1]] %*% gamma[[1]][1,]
    f.gamma[[2]]= V[[2]] %*% gamma[[2]][1,]

  # Nonparametric effects
    # see initialize betas ==0
  # Starting values of the predictors
    # just linear effects, starting values of nonpar effects ==0
      eta[[1]]   = V[[1]] %*% gamma[[1]][1,]
      eta[[2]]   = V[[2]] %*% gamma[[2]][1,]
  # Components of the Covariancematrix
    a.Sigma = prior.cov[[1]] + length(response[[1]]) #start + number of observations
  # tau2
    a.tau.sp=list()
    a.tau.rw_0 = prior.tau[1]
    b.tau.rw_0 = prior.tau[2]
    a.tau.sp_0 = prior.tau.sp[1]
    b.tau.sp_0 = prior.tau.sp[2]
    # the first parameter remains constant, no updating necessary
      a.tau.rw = a.tau.rw_0 + qr(K.rw)$rank/2
      if (geo) {
        if (numSpatial_first>0) a.tau.sp[[1]] = a.tau.sp_0 + qr(K.mrf[[1]])$rank/2
        if (numSpatial_second>0) a.tau.sp[[2]] = a.tau.sp_0 + qr(K.mrf[[2]])$rank/2
      }


############
#  GIBBS   #
############

p8=proc.time()
if (progressBar) pb = txtProgressBar(style=3)

#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
# Start of loop
for ( i in 2:(1+numBurnIn+numSamples) ){
  if (progressBar & i>51 & i%%100==0) setTxtProgressBar(pb, i/(1+numBurnIn+numSamples))

  ###############
  # Sampling of SIGMA
    Err = cbind(response[[1]]-eta[[1]]-REyes[1]*(result$Z[[1]]%*%delta[[1]][i-1,])[,,drop=T],response[[2]]-eta[[2]]-REyes[2]*(result$Z[[2]]%*%delta[[2]][i-1,])[,,drop=T])
    S = crossprod(Err)
    Sigma = rwishart(a.Sigma,chol2inv(chol( prior.cov[[2]] +S)))$IW
    sigma11[i]=Sigma[1,1]
    sigma12[i]=Sigma[1,2]
    sigma22[i]=Sigma[2,2]

  #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  # Estimation of the coefficients
    for (r in 1:2){
      cond.var[[1]]= sigma11[i]- sigma12[i]^2/sigma22[i]  #conditional variance in the first eq
      cond.var[[2]]= sigma22[i]- sigma12[i]^2/sigma11[i]  #conditional variance in the second eq
      if (r==1) offset= sigma12[i]/sigma22[i] *((response[[3-r]]-eta[[3-r]])-REyes[3-r]*(result$Z[[3-r]]%*%delta[[3-r]][i-1,])[,,drop=T])    #
      if (r==2) offset= sigma12[i]/sigma11[i] *((response[[3-r]]-eta[[3-r]])-REyes[3-r]*(result$Z[[3-r]]%*%delta[[3-r]][i-1,])[,,drop=T])    #

      #======================
      # Random Effects
        if (REyes[r]){
          b.sig_delta[[r]]= b.sig_delta_0 + 1/2 * sum(delta[[r]][i-1,]^2)   #a.sig_delta is only updated once
          sig_delta2[[r]][i,] =(rgamma(n=1, a.sig_delta[[r]], b.sig_delta[[r]]))^(-1)

          VAR.delta = solve(cond.var[[r]]^(-1)* ZZ[[r]]+diag(1/sig_delta2[[r]][i,],ncol(result$Z[[r]])))
          mu.delta= VAR.delta%*%((t(cond.var[[r]]^(-1)*result$Z[[r]])%*%(response[[r]] - (eta[[r]] +offset))))
          delta[[r]][i,] = rmvnorm(n=1, mean=mu.delta, sigma=VAR.delta,method=method)
        }
    
      #======================
      # Parametric effects
        eta_tilde= eta[[r]] - f.gamma[[r]] # ==0 if numSplines_first==0 or numSplines_second==0
        offset_tilde= offset + eta_tilde +REyes[r]*(result$Z[[r]]%*%delta[[r]][i,])[,,drop=T]

        # Updating of gamma[[r]]
        VAR.gamma[[r]] = solve(cond.var[[r]]^(-1)* VV[[r]])
        mu.gamma[[r]]= VAR.gamma[[r]]%*%((cond.var[[r]]^(-1)*t(V[[r]])%*%(response[[r]] - offset_tilde)))

        gamma[[r]][i,] = rmvnorm(n=1, mean=mu.gamma[[r]], sigma=VAR.gamma[[r]],method=method)
        f.gamma[[r]]= V[[r]] %*% gamma[[r]][i,]

        # Updating of the predictor
          eta[[r]]= eta_tilde + f.gamma[[r]]

      #======================
      # Nonparametric effects
        go=F
        if (r==1 & (numSplines_first>0|numSpatial_first>0)){go=T;npitmin=1;npitmax=numSplines_first+numSpatial_first}
        if (r==2 & (numSplines_second>0|numSpatial_second>0)){go=T;npitmin=numSplines_first+numSpatial_first+1;npitmax=length(npvars)}
        if (go)
        for (j in npitmin:npitmax){
          # spatial effect or nonparametric effect?
            if ((numSpatial_first>0 & j== numSplines_first+1) | (numSpatial_second>0 & j== numSplines_first+numSpatial_first+numSplines_second+1))
              {K= K.mrf[[r]]; a.tau= a.tau.sp[[r]]; b.tau_0=b.tau.sp_0}
            else {K= K.rw; a.tau= a.tau.rw; b.tau_0=b.tau.rw_0}

          eta_tilde= eta[[r]] - f[[j]]  #== eta[[1]] - (X_first[[j]] %*% beta_first[[j]][i-1,] -i.constraint_j[i-1])
          offset_tilde= offset + eta_tilde +REyes[r]*(result$Z[[r]]%*%delta[[r]][i,])[,,drop=T]  #

          # Updating of tau2
            #a.tau is only updated once
            b.tau= b.tau_0 + 1/2 * t(beta[[j]]) %*% K %*% beta[[j]]
            tau2 =(rgamma(n=1, a.tau, b.tau))^(-1)             
          # Updating of beta
                P= cond.var[[r]]^(-1) * XX[[j]]+ tau2^(-1) * K
                L=chol(P)
                nu= forwardsolve(L,(cond.var[[r]]^(-1) * t(X[[j]]) %*% (response[[r]]- offset_tilde)), upper.tri = TRUE, transpose = TRUE)
            mu = backsolve(L,nu)
            # beta[i-1] -> beta[i]
              beta[[j]]= mu + solve(L,rnorm(dim(X[[j]])[2],0,1))

          f[[j]]= X[[j]] %*% beta[[j]]

          # centering (varying coefficients are not centered)
            if (byvars[j]=="NA"){  #no varying coefficient
              i.constraint_j = mean(f[[j]])
              f[[j]]= f[[j]] - i.constraint_j #== f[[j]]= scale(f[[j]],scale=F)
              # compensation in intercept
                eta_tilde = eta_tilde - f.gamma[[r]]
                # Updating of "f.gamma" with the updated intercept
                  gamma[[r]][i,1] = gamma[[r]][i,1] + i.constraint_j
                  f.gamma[[r]]= V[[r]] %*% gamma[[r]][i,]
                eta_tilde = eta_tilde + f.gamma[[r]]
            }
          # Updating of eta[[r]]
            eta[[r]]= eta_tilde + f[[j]] #== eta[[r]]= eta[[r]] - (X[[j]] %*%beta[[j]][i-1,] - i.constraint_j[i-1]) + X[[j]] %*% beta[[j]][i,] - i.constraint_j[i]
          # Record Samples
            if (i>(numBurnIn+1) & (i-(numBurnIn+1))%%thin==0){
              if (byvars[j]=="NA"){
                f.sum[[j]]=f.sum[[j]]+f[[j]]
                f.rec[[j]][,fnr[[j]]]=f[[j]]
                if (!is.null(x.pred[[j]])){
                  f.pred[[j]]= X.pred[[j]] %*% beta[[j]] - i.constraint_j
                  f.pred.sum[[j]]=f.pred.sum[[j]]+f.pred[[j]]
                  f.pred.rec[[j]][,fnr[[j]]]=f.pred[[j]]
                }
              } 
              if (record.npcoef| byvars[j]!="NA") beta.rec[[j]][fnr[[j]],]=beta[[j]]
              fnr[[j]]=fnr[[j]]+1
            }
          }
      }



  #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  # Unstandardized deviance
    Dev[i]=0#log(2*pi) +crossprod(response[[1]]-eta[[1]])  +  log(2*pi*sigma2_1[i])+sigma2_1[i]^(-1)*crossprod(sigma12[i]*(response[[1]][z_aux==1]-eta[[1]][z_aux==1])-(response[[2]]-eta[[2]]))

  if (i==51)  {cat("\nEstimated time to end: ", ((proc.time()[3]-p8[3])*(numBurnIn+numSamples)/50)%/%60,"min",round(((proc.time()[3]-p8[3])*(numBurnIn+numSamples)/50)%%60,0),"sec\n");cat("\nProgress:\n "); if (progressBar) pb = txtProgressBar(style=3)}
}
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
# end of loop

#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
# Calculation of final estimators
  #*********************
  # Covariance components
    # Discarding burn-in and thinning
     result$sigma12.sample = sigma12[seq((numBurnIn+2),(numBurnIn+numSamples+1),thin) ]*scale1*scale2
     result$sigma11.sample = sigma11[seq((numBurnIn+2),(numBurnIn+numSamples+1),thin) ]*scale1^2
     result$sigma22.sample = sigma22[seq((numBurnIn+2),(numBurnIn+numSamples+1),thin) ]*scale2^2

    result$sigma11 = mean(result$sigma11.sample)
    result$sigma12 = mean(result$sigma12.sample)
    result$sigma22 = mean(result$sigma22.sample)
    result$cov   = matrix(c( result$sigma11,            result$sigma12,
                             result$sigma12, result$sigma22 ), ncol=2)

    result$ci.sigma12= quantile(result$sigma12.sample, probs= c(0.025, 0.05, 0.5, 0.95, 0.975))
    result$ci.sigma22= quantile(result$sigma22.sample, probs= c(0.025, 0.05, 0.5, 0.95, 0.975))
    result$ci.sigma11= quantile(result$sigma11.sample, probs= c(0.025, 0.05, 0.5, 0.95, 0.975))

    result$sd.sigma11=  sqrt(var(result$sigma11.sample))
    result$sd.sigma12=  sqrt(var(result$sigma12.sample))
    result$sd.sigma22=  sqrt(var(result$sigma22.sample))

  #*********************
  # Parametric effects
    # Discarding burn-in and thinning
      gamma_first.sample = as.data.frame(gamma[[1]][seq((numBurnIn+2),(numBurnIn+numSamples+1),thin), ,drop=F])  *scale1
      gamma_second.sample = as.data.frame(gamma[[2]][seq((numBurnIn+2),(numBurnIn+numSamples+1),thin), ,drop=F]) *scale2
    result$coefficients.sample=list(first=gamma_first.sample,second =gamma_second.sample)
    if (ncol(V[[1]])>0) dimnames(gamma_first.sample)[[2]]= dimnames(result$coefficients.sample[[1]])[[2]]= g_first
    if (ncol(V[[2]])>0) dimnames(gamma_second.sample)[[2]]=dimnames(result$coefficients.sample[[2]])[[2]]= g_second
    if (!is.null(gamma_second.sample[[response.first]])) gamma_second.sample[[response.first]]=result$coefficients.sample[[2]][[response.first]]=gamma_second.sample[[response.first]]/scale1
    gamma_first.mean   = colMeans(gamma_first.sample)
    gamma_second.mean   = colMeans(gamma_second.sample)
    result$coefficients=list(first=gamma_first.mean,second=gamma_second.mean)

    sd.first=apply(gamma_first.sample,2, function(x) sqrt(var(x)))
    sd.second=apply(gamma_second.sample,2, function(x) sqrt(var(x)))
    result$sd.coefficients=list(first=sd.first,second=sd.second)

    # Quantiles
      ci_gamma.first = apply(gamma_first.sample, MARGIN=2, quantile, probs= c(0.025, 0.5, 0.975))
      ci_gamma.second = apply(gamma_second.sample, MARGIN=2, quantile, probs= c(0.025, 0.5, 0.975))
      result$ci.coefficients=list(first=ci_gamma.first,second=ci_gamma.second)


    # random effects
      result$sigma.random.sample= result$sigma.random= result$coefficients$random=result$coefficients.sample$random=list()
      for (r in 1:2)
        if (REyes[r]){
          result$sigma.random.sample[[r]] = as.data.frame(sqrt(sig_delta2[[r]][seq((numBurnIn+2),(numBurnIn+numSamples+1),thin), ,drop=F]))*scalevec[r]
          result$sigma.random[[r]]  = mean(result$sigma.random.sample[[r]])
          result$coefficients.sample$random[[r]] = as.data.frame(delta[[r]][seq((numBurnIn+2),(numBurnIn+numSamples+1),thin), ,drop=F])*scalevec[r]
          result$coefficients$random[[r]]  = colMeans(result$coefficients.sample$random[[r]])
        }


  if (SCALE & length(npvars)>0){
    npcovariates=npcovariates.unscaled
    V=V.unscaled
  }

  #*********************
  # Nonparametric effects
    ci.f.pred= result$ci.f= result$f.by= result$f=result$ci.npcoef=result$npcoef=  list()
    if (length(npvars)>0){
    result$eq=numeric()
      #rescaling
      if (numSplines_first>0|numSpatial_first>0){
       result$eq=rep(1,(numSplines_first+numSpatial_first))
       for (i in 1:(numSplines_first+numSpatial_first)){
        result$eq=rep(1,(numSplines_first+numSpatial_first))
          f.sum[[i]] =f.sum[[i]] *scale1
          f.rec[[i]]=f.rec[[i]]  *scale1
          if (length(beta.rec[[i]])>1) beta.rec[[i]]=beta.rec[[i]]*scale1
          if (!is.null(x.pred[[i]])){
            f.pred.sum[[i]]=f.pred.sum[[i]]*scale1
            f.pred.rec[[i]]=f.pred.rec[[i]]*scale1
          }
        }
        }
      if (numSplines_second>0|numSpatial_second>0){
        result$eq=c(result$eq,rep(2,numSplines_second+numSpatial_second))
        for (i in (numSplines_first+numSpatial_first+1):length(npvars)){
          f.sum[[i]] =f.sum[[i]] *scale2
          f.rec[[i]]=f.rec[[i]]  *scale2
          if (length(beta.rec[[i]])>1) beta.rec[[i]]=beta.rec[[i]]*scale2
          if (!is.null(x.pred[[i]])){
            f.pred.sum[[i]]=f.pred.sum[[i]]*scale2
            f.pred.rec[[i]]=f.pred.rec[[i]]*scale2
          }
        }
       }
      #get fits
      for (j in 1:length(npvars)){
        if (length(beta.rec[[j]])>1){
          result$npcoef[[j]]   = colMeans(beta.rec[[j]])
          result$ci.npcoef[[j]]= apply(beta.rec[[j]], MARGIN=2, quantile, probs= c(0.025, 0.10, 0.90, 0.975))
        }
        else result$npcoef[[j]] = result$ci.npcoef[[j]]= 0

        if (length(f.rec[[j]])>1){
          result$f[[j]]= f.sum[[j]] /(numSamples%/%thin)
          result$ci.f[[j]]= t(apply(f.rec[[j]], MARGIN=1, quantile, probs= c(0.025, 0.10, 0.90, 0.975)))
          if (!is.null(x.pred[[j]])){
            f.pred[[j]]=f.pred.sum[[j]] /(numSamples%/%thin)
            ci.f.pred[[j]]= t(apply(f.pred.rec[[j]], MARGIN=1, quantile, probs= c(0.025, 0.10, 0.90, 0.975)))
          }
        }
        else result$f[[j]] = result$ci.f[[j]]= f.pred[[j]]=0

        if (byvars[j]!="NA"){
          result$f[[j]]=X[[j]]%*%result$npcoef[[j]]
          result$f.by[[j]]=Xby[[j]]%*%result$npcoef[[j]]
          temp.sample=apply(beta.rec[[j]], 1, function(x) Xby[[j]]%*%x)
          result$ci.f[[j]]= t(apply(temp.sample, MARGIN=1, quantile, probs= c(0.025, 0.10, 0.90, 0.975)))
          rm(temp.sample)
        }
        else result$f.by[[j]]=0
      }
      names(result$npcoef) = names(result$f) = names(result$f.by) = names(result$ci.npcoef)= names(result$ci.f)  = npvars
      if (!all(unlist(lapply(x.pred,is.null)))){
        names(f.pred)=npvars[byvars=="NA"]
        for (j in 1:length(npvars)) if (byvars[j]!="NA") names(f.pred)= paste(byvars[j],"*",npvars[j],sep="") 
      }
      for (j in 1:length(npvars)) if (byvars[j]!="NA") names(result$f)[j]=paste(byvars[j],"*",npvars[j],sep="")
    }


#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
# "CI's" for spatial effects
  if (numSpatial_first>0) {
    pcat80.first=mapply(FUN=function(x,y) {if (x<0 & y<0) return(-1)
                                         if ((x<=0 & y>=0)|(x>=0 & y<=0)) return(0)
                                         if (x>0 & y>0) return(1)},
                      t(result$ci.f[[numSplines_first+1]][,2]), t(result$ci.f[[numSplines_first+1]][,3]))
    pcat95.first=mapply(FUN=function(x,y) {if (x<0 & y<0) return(-1)
                                         if ((x<=0 & y>=0)|(x>=0 & y<=0)) return(0)
                                         if (x>0 & y>0) return(1)},
                      t(result$ci.f[[numSplines_first+1]][,1]), t(result$ci.f[[numSplines_first+1]][,4]))
  }
  else pcat80.first=pcat95.first=NULL
  if (numSpatial_second>0) {
    pcat80.second=mapply(FUN=function(x,y) {if (x<0 & y<0) return(-1)
                                         if ((x<=0 & y>=0)|(x>=0 & y<=0)) return(0)
                                         if (x>0 & y>0) return(1)},
                      result$ci.f[[numSplines_first+numSpatial_first+numSplines_second+1]][,2],result$ci.f[[numSplines_first+numSpatial_first+numSplines_second+1]][,3])
    pcat95.second=mapply(FUN=function(x,y) {if (x<0 & y<0) return(-1)
                                         if ((x<=0 & y>=0)|(x>=0 & y<=0)) return(0)
                                         if (x>0 & y>0) return(1)},
                      result$ci.f[[numSplines_first+numSpatial_first+numSplines_second+1]][,1],result$ci.f[[numSplines_first+numSpatial_first+numSplines_second+1]][,4])
  }
  else pcat80.second=pcat95.second=NULL

  result$pcat80=list(first=pcat80.first,second=pcat80.second)
  result$pcat95=list(first=pcat95.first,second=pcat95.second)

#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
# Fitted values / eta
  # First equation
    fitted.first= V[[1]]%*%result$coefficients$first
    if (numSplines_first>0 |numSpatial_first>0){
      for (j in 1:(numSplines_first+numSpatial_first))
        fitted.first = fitted.first + result$f[[j]]
    }
  # Second equation
    fitted.second= V[[2]]%*%result$coefficients$second
    if (numSplines_second>0 |numSpatial_second>0){
      for (j in 1:(numSplines_second+numSpatial_second))
        fitted.second=fitted.second + result$f[[j + (numSplines_first+numSpatial_first)]]
    }
  result$fitted.values=list(first=fitted.first,second=fitted.second)

# DIC
#  Dev.sample = Dev[seq((numBurnIn+2), (numBurnIn+numSamples+1), thin)]
#  Dev.mean = mean(Dev.sample)
  Dev.post = 0#log(2*pi) + crossprod(z.mean - fitted.first)   +   log(2*pi*result$sigma2_1) + result$sigma2_1^(-1) * crossprod(result$sigma12*(z.mean[z_aux==1]-fitted.first[z_aux==1]) - (response[[2]]-fitted.second))
  result$DIC = NA#2*Dev.mean - Dev.post


#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
# Posterior probabilities for parametric effects
  pl0.first= apply(gamma_first.sample,2,function(x) 1/length(x)*sum(x<0))
  pu0.first= apply(gamma_first.sample,2,function(x) 1/length(x)*sum(x>0))
  pl0.second= apply(gamma_second.sample,2,function(x) 1/length(x)*sum(x<0))
  pu0.second= apply(gamma_second.sample,2,function(x) 1/length(x)*sum(x>0))

  result$pvalues.onesided=list(first=data.frame(cbind(pl0.first,pu0.first),row.names=g_first),second=data.frame(cbind(pl0.second,pu0.second),row.names=g_second))
  result$pvalues.twosided=list(first=apply(result$pvalues.onesided$first,1,function(x) 2*min(x)),second=apply(result$pvalues.onesided$second,1,function(x) 2*min(x)))

p9= proc.time()
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  result$npcoef.sample=beta.rec
  if (return.f) {result$f.sample= f.rec;result$f.pred.sample= f.pred.rec}
  if (!all(unlist(lapply(x.pred,is.null)))){
   result$f.pred=f.pred
   result$ci.f.pred=ci.f.pred
  }
  result$responses=list(response[[1]],response[[2]])
  names(result$responses)=c(response.first,response.second)
  result$covariates=list(parametric=V,nonparametric=npcovariates,by=byvars.list)
  result$x.pred=x.pred
  result$formulas=list(first=first,second=second)
  result$numSplines=list(first=numSplines_first,otucome=numSplines_second)
  result$numSpatial=list(first=numSpatial_first,second=numSpatial_second)
  result$sp=list(first=sp_first,second=sp_second)
  result$numBurnIn=numBurnIn
  result$numSamples=numSamples
  result$thin=thin


#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
# OUTPUT
  if (progressBar) close(pb)
  cat("\n\nTotal time elapsed\n");print(p9-p0)
  class(result)=c("bayesIV","bayesIVgauss")

  summary.bayesIV(result)
  #scbs
    if (length(result$covariates$nonparametric)>0 & scbs) result$scb=plot.bayesIV(result,scb=TRUE,scb.level=scb.level,scb.method=scb.method,PLOT=F)
  # Plot sampling paths of parametric effects
    if (plotpaths & (ncol(V[[1]])>0 | ncol(V[[2]])>0))
        plotPaths(cbind(gamma[[1]], gamma[[2]]), names=c(g_first,g_second),numBurnIn=numBurnIn)#,style=plotpaths.style , output=ifelse(output=="window",yes="window",no=paste(output,date,"SamplingPaths.pdf"))

  if (!return.samples) result$sigma12.sample=result$sigma22.sample=result$sigma11.sample=result$coefficients.sample=result$npcoef.sample= NULL
  invisible(result)
}

