# plot normal mixture marginal and bivariate densities. Based on plot.bayesm.nmix in package bayesm.
density.bayesIV=function(object,
marginal=TRUE,contour=T,persp=T,color=F,   contour.col=terrain.colors(100), persp.col="green3",
theta = -30, phi = 15, ltheta = 120,expand=0.9, shade = 0.75, border = NULL, box = FALSE,
nlevels= 10,
nstd=2,ngrid=50, ndraw, Grid,
tol=0.001, ask=T,
#mfrow=c(1,1),
#marginal.col="magenta",
...){


  if (class(object)[2]!="bayesIV") stop("Only objects created by bayesIV() supported.")
    op=par(no.readonly=TRUE)
    on.exit(par(op))
    
  if (class(object)[3]=="jara"){
    x=object$density

    if(ask==T) par(ask=dev.interactive())
 #   if (is.numeric(par)) par(mfrow=mfrow)

    # rescale
      grid1=x$grid1[min(which((x$fun1>=tol))):max(which((x$fun1>=tol)))]*x$scale1 #+x$mu1
      grid2=x$grid2[min(which((x$fun2>=tol))):max(which((x$fun2>=tol)))]*x$scale2 #+x$mu2
      f=x$f[min(which((x$fun1>=tol))):max(which((x$fun1>=tol))),min(which((x$fun2>=tol))):max(which((x$fun2>=tol)))]
      fun1=x$fun1[min(which((x$fun1>=tol))):max(which((x$fun1>=tol)))]
      fun2=x$fun2[min(which((x$fun2>=tol))):max(which((x$fun2>=tol)))]

    # marginal densities
    if (marginal){
      plot(grid1,fun1/x$scale1,lwd=2,type="l",xlab=expression(epsilon[1]),ylab="Density")
      plot(grid2,fun2/x$scale2,lwd=2,type="l",xlab=expression(epsilon[2]),ylab="Density")
     }

    # bivariate density
      xx<-matrix(grid1,ncol=1); colnames(xx)<-x$varnames[1]
      yy<-matrix(grid2,ncol=1); colnames(yy)<-x$varnames[2]
      z<-f

     title1<-paste("Predictive Density of ",paste(x$varnames[1],x$varnames[2],sep="-"),sep='')

     if (contour){
      if (color) image(xx,yy,z,col=contour.col,xlab=expression(epsilon[1]),ylab=expression(epsilon[2]),...)      #    main=title1,xlab=x$varnames[1],ylab=x$varnames[2]
      contour(xx,yy,z,add=color,drawlabels=FALSE,xlab=expression(epsilon[1]),ylab=expression(epsilon[2]),nlevels=nlevels,...)    # ,main=title1,xlab=x$varnames[1],ylab=x$varnames[2]
     #   contour(xx,yy,z,main=title1,xlab=x$varnames[1],ylab=x$varnames[2])
     }
      if (persp) persp(xx,yy,z,xlab=x$varnames[1],ylab=x$varnames[2],zlab="density",theta=theta,phi=phi,expand = expand, ltheta = ltheta,main=title1,col = ifelse(color,persp.col,"white"),  shade = color*shade, border = border, box = box)


    invisible(list(grid1=grid1,grid2=grid2,f=f,fun1=fun1/x$scale1,fun2=fun2/x$scale2))
  }
  else{
  names=c(expression(epsilon[1]),expression(epsilon[2]))
    nmixlist=object$nmix
    if(mode(nmixlist) != "list") stop(" Argument must be a list \n")
    probdraw=nmixlist[[1]]; compdraw=nmixlist[[3]]
    if(!is.matrix(probdraw)) stop(" First element of list (probdraw) must be a matrix \n")
    if(mode(compdraw) != "list") stop(" Third element of list (compdraw) must be a list \n")
    numSamples=length(probdraw)
    if(numSamples < 100) {stop(" fewer than 100 draws submitted \n")}
    if (missing(ndraw)) ndraw=numSamples

    datad=length(compdraw[[1]][[1]]$mu)
    if(missing(names)) {names=as.character(1:datad)}
    ind=as.integer(seq(from=1,to=numSamples,length.out=max(ndraw,trunc(.05*numSamples))))

    if(missing(Grid)){
      Grid=matrix(0,nrow=ngrid,ncol=datad)
        out=momMix2(probdraw[ind,,drop=FALSE],compdraw[ind],scale1=object$density$scale1,scale2=object$density$scale2)
        mu=out$mu
        sd=out$sd
        for(i in 1:datad ) Grid[,i]=c(seq(from=(mu[i]-nstd*sd[i]),to=(mu[i]+nstd*sd[i]),length=ngrid))
    }

    if(ask==T) par(ask=dev.interactive())
#    if (is.numeric(par)) par(mfrow=mfrow)

    # plot posterior mean of marginal densities
      if(marginal){
        mden=eMixMargDen2(Grid,probdraw[ind,,drop=FALSE],compdraw[ind],scale1=object$density$scale1,scale2=object$density$scale2)
        nx=datad
        for(index in 1:nx){
          plot(range(Grid[,index]),c(0,1.1*max(mden[,index])),type="n",xlab="",ylab="density",...)
          title(names[index])
            lines(Grid[,index],mden[,index],col="black",lwd=2,...)
#            polygon(c(Grid[1,index],Grid[,index],Grid[nrow(Grid),index]),c(0,mden[,index],0),col=marginal.col,...)
          
        }
      }

    # now plot bivariates in list
      if(contour|persp){
        bi.sel=list(c(1,2))
        nsel=length(bi.sel)
        den=array(0,dim=c(ngrid,ngrid,nsel))
        lstxixj=NULL
        for(sel in 1:nsel){
          i=bi.sel[[sel]][1]
          j=bi.sel[[sel]][2]
          xi=Grid[,i]
          xj=Grid[,j]
          lstxixj[[sel]]=list(xi,xj)
          for(elt in ind){
            den[,,sel]=den[,,sel]+mixDenBi2(i,j,xi,xj,probdraw[elt,,drop=FALSE],compdraw[[elt]],scale1=object$density$scale1,scale2=object$density$scale2)
          }
          den[,,sel]=den[,,sel]/sum(den[,,sel])
        }
        for(index in 1:nsel){
          xi=unlist(lstxixj[[index]][1])
          xj=unlist(lstxixj[[index]][2])
          xlabtxt=names[bi.sel[[index]][1]]
          ylabtxt=names[bi.sel[[index]][2]]
          if (contour){
            if (color) image(xi,xj,den[,,index],col=contour.col,xlab=xlabtxt,ylab=ylabtxt,...)
            contour(xi,xj,den[,,index],add=TRUE,drawlabels=FALSE,nlevels=nlevels,...)
          }
          if (persp) persp(xi,xj,den[,,index], theta = theta, phi = phi, col = persp.col,scale = F, ltheta = ltheta,expand=expand, shade = shade, border = border, box = box,...)   #     expand=10,
        }
      }
    invisible(list(x=xi,y=xj,z=den[,,index]))
  }
}

