#compute simultaneous credible bands
  scbB <- function(object,select, level=0.95,method="TK",center=T){
    if (class(object)[1]!="bayesIV") stop ("This summary function can be used only with a bayesIV object")
    samples <- t(object$f.sample[[select]])
    if (!center){
      #if (class(object)[2]=="bayesIV")  samples=matrix(mapply(FUN=function(f,int) f+int, samples,object$int.sample[,object$eq[select]],SIMPLIFY =T),nrow=nrow(samples))
      #else
      samples=matrix(mapply(FUN=function(f,int) f+int, samples,object$coefficients.sample[[object$eq[select]]][,1],SIMPLIFY =T),nrow=nrow(samples))
    }

    if (method=="JB") {      ## CB Besag
      n <- nrow(samples)
      p <- ncol(samples)

      samples.sorted <- matrix(0, n, p)
      samples.rank <- matrix(0, n, p)
      for(i in 1:p){
        samples.sorted[,i] <- sort(samples[,i])
        samples.rank[,i] <- rank(samples[,i])
      }

      k <- trunc(level*n)+1
      helpset <- rep(0, n)
      for(i in 1:n) helpset[i] <- max(c(n+1-min(samples.rank[i,]), max(samples.rank[i,])))
      helpset <- sort(helpset)
      t.star <- helpset[k]

      lower <- samples.sorted[n+1-t.star,]
      upper <- samples.sorted[t.star,]
    }
    else if (method=="CC"){         #CB Ciprian Crainiceanu
      n <- nrow(samples)
      p <- ncol(samples)

      pmean <- apply(samples, 2, mean)
      psd <- apply(samples, 2, sd)

      samples.standard <- matrix(0, n, p)
      for(i in 1:n) samples.standard[i,] <- (samples[i,]-pmean)/psd

      helpset <- rep(0, n)
      for(i in 1:n) helpset[i] <- max(abs(samples.standard[i,]))

      m.level <- quantile(helpset, probs=level)
      lower <- pmean - m.level*psd
      upper <- pmean + m.level*psd
    }
    else if (method=="TK"){             #CB Thomas Kneib
      # at least k functions completely cotained in the band
      n <- nrow(samples)
      p <- ncol(samples)

      k <- trunc(level*n)+1

      pmean <- apply(samples, 2, mean)
      pqlow <- apply(samples, 2, quantile, probs=(1-level)/2)
      pqupp <- apply(samples, 2, quantile, probs=1-(1-level)/2)

      count <- function(x) sum(x>upper) + sum(x<lower)

      fac1 <- 1
      lower <- pqlow
      upper <- pqupp
      k1 <- sum(apply(samples, 1, count) == 0)

      fac2 <- 1.5
      lower <- pmean + fac2*(pqlow-pmean)
      upper <- pmean + fac2*(pqupp-pmean)
      k2 <- sum(apply(samples, 1, count) == 0)

      while(k2 < k){
        fac2 <- 1.5*fac2
        lower <- pmean + fac2*(pqlow-pmean)
        upper <- pmean + fac2*(pqupp-pmean)
        k2 <- sum(apply(samples, 1, count) == 0)
      }

      while(k1+1 < k2){
        facnew <- 0.5*(fac1+fac2)
        lower <- pmean + facnew*(pqlow-pmean)
        upper <- pmean + facnew*(pqupp-pmean)
        knew <- sum(apply(samples, 1, count) == 0)

        if(knew < k){
          k1 <- knew
          fac1 <- facnew
        }
        if(knew >=k){
          k2 <- knew
          fac2 <- facnew
        }
      }
    }
    else stop ("unknown method")

    fitted=object$f[[select]]
    covariate=matrix(object$covariates$nonparametric[[select]])
    dimnames(covariate)=dimnames(fitted)= list(NULL,names(object$f)[select])
    scbb<-(list(covariate=covariate,fitted=fitted,lower=lower, upper=upper,center=center))
    class(scbb)<-"scbb"
    return(scbb)
  }
