############################
# Summary
  summary.bayesIV=function(object, digits=5){
  if (class(object)[1]!="bayesIV") stop ("This summary function can be used only with a bayesIV object")
if (class(object)[2]=="ssm"){

  cat("\nFormulas: \n")
  cat(" Selection equation: ")
  print(object$formulas$selection)
  cat(" Outcome equation:   ")
  print(object$formulas$outcome)
  cat("\n",length(object$responses$z)," observations (",sum(object$responses$z==0)," censored and ",sum(object$responses$z==1)," observed)" , sep = "")
  cat("\nNumber of samples discarded (burn-in): ",object$numBurnIn,"; Number of Samples used: ",floor(object$numSamples/object$thin)," of ",object$numSamples," Samples (thinning=",object$thin,")\n", sep = "")

  #parametric coefficients
    cat("\nParametric coefficients:\n")
    output.sel = as.data.frame(matrix(c(round(object$coefficients[[1]],digits),
                                        round(object$coefficients.glm,digits),
                                        round(object$coefficients.mgcv[[1]],digits),
                                        round(object$sd.coefficients[[1]],digits),
                                        round(t(object$ci.coefficients[[1]]),digits),
                                        round(as.matrix(object$pvalues.onesided[[1]]),digits),
                                        round(object$pvalues.twosided[[1]],digits)

                                        ),
                                      ncol=10, byrow =F )
                               , row.names =names(object$coefficients[[1]]))
    dimnames(output.sel)[[2]] = c("Estimate","glm-estimate","mgcv-estimate","Std.Dev.","2.5%quant.","Median","97.5%quant.","Pr(<0)","Pr(>0)","p-value(2sided)")
    output.out = as.data.frame(matrix(c(round(object$coefficients[[2]],digits),
                                        round(object$coefficients.lm,digits),
                                        round(object$coefficients.mgcv[[2]],digits),
                                        round(object$sd.coefficients[[2]],digits),
                                        round(t(object$ci.coefficients[[2]]),digits),
                                        round(as.matrix(object$pvalues.onesided[[2]]),digits),
                                        round(object$pvalues.twosided[[2]],digits)
                                        ),
                                      ncol=10, byrow =F ),
                               row.names = names(object$coefficients[[2]]))
    dimnames(output.out)[[2]] = c("Estimate","OLS-estimate","mgcv-estimate","Std.Dev.","2.5%quant.","Median","97.5%quant.","Pr(<0)","Pr(>0)","p-value(2sided)")

    cat("\nSelection equation:\n")
    print(output.sel)
    cat("\nOutcome equation:\n")
    print(output.out)

  cat("\nSIGMA: \n")
  print(matrix(round(object$cov,digits), ncol=2))
  cat("\n")

  output.var= as.data.frame(matrix(c(round(object$sigma12,digits),
                                     round(object$sd.sigma12,digits),
                                     round(object$ci.sigma12,digits),
                                     round(object$sigma2_1+object$sigma12^2,digits),
                                     round(object$sd.sigma22,digits),
                                     round(object$ci.sigma22,digits),
                                     round(object$corr,digits),
                                     round(object$sd.corr,digits),
                                     round(object$ci.corr,digits)
                                    ),
                                  ncol=7, byrow =T ),
                             row.names = c("cov(eps1,eps2)","Var(eps2)","Correlation")) # "Correlation between the disturbances"
  dimnames(output.var)[[2]] = c("Mean","Std.Dev.","2.5","5%","50%","95%","97.5%")
  cat("Quantiles of the covariance components and the correlation between disturbances:\n")
  print(output.var)

  cat("\nDIC:", round(object$DIC, digits),"\n\n")
}
else{
     cat("\nFormulas: \n")
    cat(" First equation: ")
    print(object$formulas$first)
    cat(" Second equation:   ")
    print(object$formulas$second)

    cat("\n",length(object$responses[[1]])," observations" , sep = "")
    cat("\nNumber of samples discarded (burn-in): ",object$numBurnIn,"; Number of Samples used: ",floor(object$numSamples/object$thin)," of ",object$numSamples," Samples (thinning=",object$thin,")\n", sep = "")
    if (class(object)[2]=="bayesIVgauss") cat("\nBivariate Gaussian errors assumed\n")
    else  cat("\nDistribution of error terms flexibly estimated\n")
    #parametric coefficients
      cat("\nParametric coefficients:\n")
      if (length(object$coefficients$first)>0){
        output.first = as.data.frame(matrix(c(round(object$coefficients[[1]],digits),
                                            round(object$coefficients.lm_first,digits),
                                            round(object$coefficients.mgcv[[1]],digits),
                                            round(object$sd.coefficients[[1]],digits),
                                            round(t(object$ci.coefficients[[1]]),digits),
                                            round(as.matrix(object$pvalues.onesided[[1]]),digits),
                                            round(object$pvalues.twosided[[1]],digits)
                                            ),
                                          ncol=10, byrow =F )
                                   , row.names =names(object$coefficients[[1]]))
      dimnames(output.first)[[2]] = c("Estimate","OLS","mgcv","Std.Dev.","2.5%quant.","Median","97.5%quant.","Pr(<0)","Pr(>0)","p-value(2sided)")
      }
      if(length(object$coefficients$second)>0) {
      output.second = as.data.frame(matrix(c(round(object$coefficients[[2]],digits),
                                          round(object$coefficients.lm_second,digits),
                                          round(object$coefficients.mgcv[[2]],digits),
                                          round(object$sd.coefficients[[2]],digits),
                                          round(t(object$ci.coefficients[[2]]),digits),
                                          round(as.matrix(object$pvalues.onesided[[2]]),digits),
                                          round(object$pvalues.twosided[[2]],digits)
                                          ),
                                        ncol=10, byrow =F ),
                                 row.names = names(object$coefficients[[2]]))
      dimnames(output.second)[[2]] = c("Estimate","OLS","mgcv","Std.Dev.","2.5%quant.","Median","97.5%quant.","Pr(<0)","Pr(>0)","p-value(2sided)")
      }
      cat("\nFirst equation:\n")
      if (length(object$coefficients$first)>0) if(length(object$coefficients[[1]])>0) print(output.first)
      cat("\nSecond equation:\n")
      if(length(object$coefficients$second)>0) print(output.second)

    if (class(object)[2]=="bayesIVgauss"){
      cat("\nSIGMA: \n")
      print(matrix(round(object$cov,digits), ncol=2))
      cat("\n")

      output.var= as.data.frame(matrix(c(round(object$sigma11,digits),
                                         round(object$sd.sigma11,digits),
                                         round(object$ci.sigma11,digits),
                                         round(object$sigma12,digits),
                                         round(object$sd.sigma12,digits),
                                         round(object$ci.sigma12,digits),
                                         round(object$sigma22,digits),
                                         round(object$sd.sigma22,digits),
                                         round(object$ci.sigma22,digits)
                                        ),
                                      ncol=7, byrow =T ),
                                 row.names = c("Var(eps1)","cov(eps1,eps2)","Var(eps2)")) # "Correlation between the disturbances"
      dimnames(output.var)[[2]] = c("Mean","Std.Dev.","2.5","5%","50%","95%","97.5%")
      cat("Quantiles of the covariance components:\n")
      print(output.var)
    }
    cat("\nDIC:", round(object$DIC, digits),"\n\n")
}


  }
