\name{am}
\alias{am}
\title{Fitting Bayesian Geoadditive Models}
\description{
Fits a Bayesian semiparametric regression model.
Inference is fully Bayesian. Nonparametric effects are modeled with P-splines. Also geoadditive effects and
varying coefficients can be fitted.
}
\usage{
am(form, random = list(NULL), data,
            numBurnIn = 2000, numSamples = 20000, thin = 20,
            numKnots = 20, degree = 3, rw.order = 2,%knotchoice = "equidistant",
            contrasts = NULL, graph = NULL, x.pred = NULL,
            prior.cov = c(0.001, 0.001),
            prior.random = c(0.001, 0.001), prior.tau = c(0.001, 0.001),
            prior.tau.sp = c(0.001, 0.001),
            seed = NULL, scbs = FALSE, scb.level = 0.95, scb.method = "TK",
            record.npcoef = FALSE, return.samples = T, return.f = T,
            mgcv = FALSE, plotpaths = FALSE, progressBar = T, method = "chol")
}
\arguments{
\item{form}{the formula.
    Specification of formulas is similar to package \code{mgcv}, with an additional possibility to fit spatial
    effects specified with \code{sp(.)} (smooth functions
    are specified with \code{s(.)} and varying coefficients with
    \code{s(., by=.)}). If nonparametric effects are included, an
    intercept is obligatory, otherwise exclusion of the intercept with
    -1 and +0, as usual.}
\item{random}{list of cluster IDs for random intercepts. Must be given as factors.}
\item{data}{a data frame containing the variables in the model.}
\item{graph}{a \code{\link{BayesX}} graph object containing information of neighborhood structures for spatial effects.}
\item{contrasts}{optional lists for the specification of contrasts in the case of catergorical covariates. See \code{contrasts.arg} of \code{\link{model.matrix.default}}.}
\item{numBurnIn, numSamples}{number of iterations in burn-in period that will be discarded and number of samples that will be used for estimation, respectively.}
\item{thin}{the degree of thinning. If 1, every sample after the burn-in period is used.}
\item{numKnots}{number of B-spline basis functions for nonparametric effects.}
\item{degree}{the degree of the B-spline basis functions according to function bs() in package \code{splines}, default is 3 for cubic splines.}
\item{rw.order}{the order of the random walk penalty matrices of smooth functions. }
\item{x.pred}{(optional) list of vectors of covariates for which predicted function values are returned. }
\item{prior.cov}{shape and scale parameters for the inverse Gamma prior on the residual variance}
\item{prior.random}{shape and scale parameters for the inverse Gamma prior on the variance of a possible random effect}
\item{prior.tau}{shape and scale parameters for the inverse Gamma prior on the variance parameters which influence the smoothness of curves.      }
\item{prior.tau.sp}{shape and scale parameters for the inverse Gamma prior on the variance parameters which influence the smoothness of spatial effects.}
\item{seed}{a seed for set.seed (optional).}
\item{record.npcoef}{record samples of the regression coefficients of nonparametric effects? Returned only if return.samples=TRUE.  }
\item{return.samples}{return matrices of samples of all estimates (including components of the covariance matrix, nonparametric (only if record.beta=T) and parametric estimates)? }
\item{return.f}{Specify if fitted values for every smooth effect of each Gibbs iteration are to be returned. }
\item{mgcv}{Specify if univariate estimation using package mgcv is included for comparison. Spatial effects are discarded.}
\item{scbs}{if TRUE, simultaneous credible bands are computed}
\item{scb.level}{level of simultaneous credible bands}
\item{scb.method}{method for simultaneous credible band: "TK" (Thomas Kneib), "CC" (Ciprian Crainiceanu) or "JB" (Besag)}
\item{method}{}
\item{plotpaths}{logical, if TRUE the sampling paths are plotted  }
\item{progressBar}{show txtProgressBar during simulation}
}
\details{
%%  ~~ If necessary, more details than the description above ~~
}
\value{
\item{coefficients.sample, npcoef.sample}{MCMC samples of the parameters after the burn-in period and after thinning.   }
\item{f.sample}{MCMC samples of the nonparametric functions.   }
\item{f, f.mgcv}{function values evaluated at the designpoints for Bayesian estimation and frequentist estimation using \code{\link[mgcv]{gam}}.   }
\item{f.by}{function values of functions with varying coefficients evaluated at the designpoints}
\item{ci.f}{the corresponding pointwise credible intervals.   }
\item{f.pred, f.mgcv.pred, ci.f.pred}{predicted function values evaluated at points given by \code{x.pred} for Bayesian estimation and frequentist estimation using \code{\link[mgcv]{gam}} and the corresponding credible intervals.   }
\item{pcat80, pcat95}{for the visualisation of 80\% and 95\% posterior probabilities of geoadditive effects. Regions with significantly positive, insignificant and significantly negative values are coded with +1, 0 and -1, respectively.   }
\item{coefficients,coefficients.mgcv,coefficients.glm,coefficients.lm}{coefficients in the case of Bayesian estimation, frequentist estimation obtained by \code{\link[mgcv]{gam}} and frequentist estimation obtained by \code{\link{lm}}.   }
\item{sd.coefficients,sd.coefficients.mgcv}{the standard deviations of the coefficients }
\item{ci.coefficients}{ 90\% and 95\% credible intervals of the parametric effects }
\item{npcoef,ci.npcoef}{coefficients and credible intervals of the coefficients of nonparametric effects}
\item{pvalues.onesided}{onesided Bayesian p-values.}
\item{pvalues.twosided}{two-sided Bayesian p-values.}
\item{responses}{the response vector}
\item{covariates}{the included covariates}
\item{x.pred}{the grid points}
\item{fitted.values}{the fitted values.}
\item{formulas, numSplines, numSpatial, numBurnIn, numSamples, thin}{arguments given in the function call.   }
}
\references{
%% ~put references to the literature/web site here ~
}
\author{
Manuel Wiesenfarth}
\note{
%%  ~~further notes~~
}
\seealso{
\code{\link{bayesIV}} and \code{\link{bayesIVgauss}} for estimation in models with one endogenous regressor; package \code{\link{BayesX}} for description of spatial effects related functions.
}
\examples{
#see bayesIV()
}