\name{bayesIV}
\alias{bayesIV}
\title{ Fitting Bayesian Semiparametric Instrumental Variable Regression models with Non-Normal Errors}
\description{
Fits a Bayesian semiparametric instrumental variable regression model with flexible bivariate error distribution.
Inference is fully Bayesian. Nonparametric effects are modeled with P-splines. Also geoadditive effects and
varying coefficients can be fitted.
}
\usage{
bayesIV(first, second, random=list(NULL,NULL), data,
numBurnIn=2000, numSamples=20000, thin=20, numKnots=40, degree=3, rw.order=2,
contrasts.first =NULL, contrasts.second =NULL, graph= NULL, x.pred=NULL,
density.method="jara",
priorDPM =
  list(
    # fixed parameters in G0
      tau.Sigma=.0036, s.Sigma=3, S.Sigma=0.2*diag(2),
    # hyperpriors on parameters in G0 (only density.method="jara")
      a.Sigma=NULL,b.Sigma=NULL,
      s.S.Sigma=NULL,S.S.Sigma=NULL,
    # parameters for alpha discretely distributed (only density.method="conley")
      power=.8,  maxuniq=200,  Istarmin=1, Istarmax=NULL, ngrid.alpha=NULL,
    # parameters for alpha Gamma distributed (only density.method="jara")
      a.alpha=2,b.alpha=2
  ),
prior.random= c(0.001,0.001), prior.tau=c(0.001,0.001), prior.tau.sp=c(0.001,0.001),
grid.density=NULL, ngrid.density=1000,
scbs=FALSE,scb.center=TRUE,scb.level=0.95,
record.npcoef=FALSE, return.samples=TRUE, return.f=TRUE,
mgcv=TRUE, plotpaths=FALSE, progressBar=TRUE, SCALE=TRUE, seed=NULL, method="chol")
}

\arguments{
\item{first, second}{the formulas of first and second equation.
    Specification of formulas is similar to package \code{mgcv}, with an additional possibility to fit spatial
    effects specified with \code{sp(.)} (smooth functions are specified with \code{s(.)} and varying coefficients with
    \code{s(., by=.)}). Numeric variables have to be specified first, categorical variables last.}
\item{random}{list of cluster IDs for random intercepts in the two equations. Must be given as factors.}
\item{data}{a data frame containing the variables in the model. Might cause problems if data frame is not called "data" in some instances.}
\item{graph}{a \code{\link{BayesX}} graph object containing information of neighborhood structures for spatial effects.}
\item{contrasts.first, contrasts.second}{optional lists for the specification of contrasts in the case of catergorical covariates in first and second equation, respectively. See \code{contrasts.arg} of \code{\link{model.matrix.default}}.}
\item{numBurnIn, numSamples}{number of iterations in burn-in period that will be discarded and number of samples that will be used for estimation, respectively.}
\item{thin}{the degree of thinning. If 1, every sample after the burn-in period is used.}
\item{numKnots}{number of knots for nonparametric effects.}
\item{degree}{the degree of the B-spline basis functions according to function bs() in package \code{splines}, default is 3 for cubic splines.}
\item{rw.order}{the order of the random walk penalty matrices of smooth functions. }
\item{x.pred}{(optional) list of vectors of covariates for which predicted function values are returned. }
\item{density.method}{"jara" for the error density estimation using DPpackage (preferred default). "conley" for density estimation as in bayesm package. Affects possibilities of prior choices in priorDPM.}
\item{priorDPM}{a list of hyperparameters for the DPM prior on the error distribution.
The list includes the following parameters:\\
\code{s.Sigma} and \code{S.Sigma} for the hyperparameters of the inverted Wishart part of the baseline distribution,\\
\code{tau.Sigma} for the scale parameter of the normal part of the baseline distribution.\\
%
%\code{alpha} giving the value of the precision parameter (it must be specified if \code{a.alpha} is missing),
%
Only \code{density.method="jara"}  (precision parameter alpha a priori gamma distributed):\\
\code{a.alpha} and \code{b.alpha} giving the hyperparameters for prior distribution of the precision parameter alpha of the Dirichlet process prior.\\
%
Optional (set to \code{NULL} if not required): \code{a.Sigma} and \code{b.Sigma} giving the hyperparameters for the gamma prior distribution of the scale parameter \code{tau.Sigma} of the normal
part of the baseline distribution (overwrites fixed \code{tau.Sigma}),
 \code{s.S.Sigma} and \code{S.S.Sigma} giving the hyperparameters of the inverted Wishart prior distribution for the scale matrix,
\code{S.Sigma} of the inverted Wishart part of the baseline distribution (overwrites \code{S.Sigma})\\
%
Only \code{density.method="conley"} (precision parameter alpha a priori discretely distributed):\\
\code{maxuniq} for the maximum number of unique mixture compontents,
\code{ngrid.alpha} for the gridsize for alpha draws,
\code{Istarmin}, \code{Istarmax} and \code{power} see documentation of \code{rivDP} in package \code{bayesm}.
}
%
%\item{prior.mu}{mean of the prior bivariate Gaussian base distribution}
%\item{prior.cov}{covariance matrix of the prior bivariate Gaussian base distribution}
%\item{power}{}
%\item{maxuniq}{maximum number of unique theta values}
%\item{ngrid.alpha}{the gridsize for alpha draws}
%\item{Istarmax}{}
%
\item{prior.random}{shape and scale parameters for the inverse Gamma prior on the variances of possible random effects.}
\item{prior.tau}{shape and scale parameters for the inverse Gamma prior on the variance parameters which influence the smoothness of curves.}
\item{prior.tau.sp}{shape and scale parameters for the inverse Gamma prior on the variance parameters which influence the smoothness of spatial effects.}
\item{seed}{a seed for set.seed (optional).}
\item{record.npcoef}{record samples of the regression coefficients of nonparametric effects? Returned only if return.samples=TRUE.  }
\item{return.samples}{return matrices of samples of all estimates (including components of the covariance matrix, nonparametric (only if record.beta=TRUE) and parametric estimates)? }
\item{return.f}{Specify if fitted values for every smooth effect of each Gibbs iteration are to be returned. }
\item{mgcv}{Specify if univariate estimation using package mgcv is included for comparison. Spatial effects are discarded.}
\item{scbs}{if TRUE, simultaneous confidence bands are computed}
\item{scb.level}{confidence level of simultaneous confidence bands}
\item{scb.center}{TRUE for simultaneous credible bands centered around zero. FALSE to include the intercept into the simultaneous credible bands of all curves.}
%\item{scb.method}{method for simultaneous confidence band: "TK" (Thomas Kneib), "CC" (Ciprian Crainiceanu) or "JB" (Besag)}
\item{plotpaths}{logical, if TRUE the sampling paths are plotted  }
%\item{SCALE}{TRUE to scale responses. Recommended.}
\item{progressBar}{show txtProgressBar during simulation}
\item{method}{method for matrix inversion in the parametric effects Gibbs sampling block.}
%\item{knotchoice}{Specify the way knots are located. Either "equidistant" or "quantiles".}
%\item{output}{  Device on which plots are printed. Default is "window", otherwise pdf files are written with file name starting with what is given here.}
%\item{add.mgcv}{  Add smooth functions estimated by \code{mgcv} to the plots of nonparametric effects. Doesn't work with varying coefficients.       }
%\item{plotnp}{  Logical, if TRUE nonparametric effects are plotted  }
%\item{plotmaps}{  Logical, if TRUE the maps(s) with spatial effect(s) is/are plotted}
%\item{cols}{Color scheme to be employed for plotting spatial maps. Could be either a vector of colors or one out of the following pre-defined schemes: hcl, hsv, grey}
%\item{plotacf}{  Plot autocorrelation functions? If nonparametric effects are included autocorrelation functions of the first and 10th knot are plotted, but only if record.beta==TRUE.              }
%\item{lag}{  Specify the lag of the autocorrelation functions.    }
%  \item{plotmgcv}{}
%  \item{plotpaths.style}{}
}
\details{
Fits a semiparametric instrumental variable model that allows to correct for endogeneity bias in
regression models with one endogenous regressor where the covariate effects enter with unknown functional form.
Bias correction relies on a simultaneous equations specification with flexible modeling of the joint error distribution.
The structural and instrumental variables equation are specified based on geoadditive predictors comprising penalised splines
for nonlinear effects of continuous covariates, Gaussian Markov random fields for spatial effects, and a Dirichlet process prior to
deal with flexible estimation of the distribution of the error terms. Inference is fully Bayesian, employing efficient
Markov Chain Monte Carlo simulation techniques. The resulting posterior samples allow the construction of simultaneous credible bands for the nonparametric effects.
A further advantage includes the automatic smoothing parameter selection.
%For comparison with univariate regression, linear estimates are given in columns "glm" and "lm"
%calculated by functions \code{glm(family="probit")} and \code{lm}, respectively (note that these
%estimates are calculated assuming all effects linear -- those specified nonparamtrically, too --
%and excluding possible spatial effects). If \code{mgcv==TRUE}, parametric estimates calculated with
%function \code{gam} of package \code{mgcv} are given in the columns aside (nonparametric effects
%are calculated and plotted, but again excluding spatial effects).

%For spatial effects a .gra file and a .bnd file are needed describing the neighborhood structure and boundary information. See package BayesX for details.

The output includes resulting estimates, as well as the corresponding credible intervals and
posterior probabilities.

%For model first the DIC (Deviance Information Criterion) which is based on the joint likelihood, is calculated.
It is recommended to choose large numbers of numBurnIn, numSamples and thin and to check sample autocorrelations when instruments are weak.

Uses functions of package \code{bayesm} for the Dirichlet process mixture part with minor changes.
}
\value{
\item{coefficients.sample, npcoef.sample}{MCMC samples of the parameters after the burn-in period and after thinning.   }
\item{f.sample}{MCMC samples of the nonparametric functions.   }
\item{f, f.mgcv}{Function values evaluated at the designpoints for the simultaneous equations model and separate univariate regressions using \code{\link[mgcv]{gam}}.   }
\item{f.by}{Function values of functions with varying coefficients evaluated at the designpoints}
\item{ci.f}{The corresponding pointwise credible intervals.   }
\item{f.pred, f.mgcv.pred, ci.f.pred}{Predicted function values evaluated at points given by \code{x.pred} for the joint model and separate univariate regressions using \code{\link[mgcv]{gam}} and the corresponding credible intervals.   }
\item{pcat80, pcat95}{For the visualisation of 80\% and 95\% posterior probabilities of geoadditive effects. Regions with significantly positive, insignificant and significantly negative values are coded with +1, 0 and -1, respectively.   }
\item{coefficients,coefficients.mgcv,coefficients.glm,coefficients.lm}{Coefficients in the simultaneous equations model, separate univariate regressions obtained by \code{\link[mgcv]{gam}} and separate univariate parametric regressions obtained by \code{\link{lm}}.   }
\item{sd.coefficients,sd.coefficients.mgcv}{The standard deviations of the coefficients }
\item{ci.coefficients}{ 90\% and 95\% credible intervals of the parametric effects }
\item{npcoef,ci.npcoef}{coefficients and credible intervals of the coefficients of nonparametric effects}
\item{pvalues.onesided}{Onesided Bayesian p-values.}
\item{pvalues.twosided}{Two-sided Bayesian p-values.}
\item{responses}{The responses}
\item{covariates}{The included covariates}
\item{x.pred}{the grid points}
\item{fitted.values}{The fitted values.}
\item{formulas, numSplines, numSpatial, numBurnIn, numSamples, thin}{Arguments given in the function call.   }
%\item{cov, sigma12, sigma2_1, corr}{The covariance matrix of the errors, the associated covariance sigma12, conditional variance $sigma_{2|1}^2$ and correlation.}
%\item{ci.sigma12, ci.sigma22, ci.corr}{90\% and 95\% credible intervals of the covariance, the error in the second equation and the correlation between the errors based on the respective quantiles of the samples. }
%\item{sd.sigma12, sd.sigma22, sd.corr}{The respective standard deviations.}
%\item{DIC}{Deviance Information Criterion for model selection.}
}
\references{
%% ~put references to the literature/web site here ~
}
\author{Manuel Wiesenfarth}
\note{
%%  ~~further notes~~
}


\seealso{
\code{\link{rivDP}} (package bayesm) with similar implementation (in case of \code{density.method="conley"}) for parametric covariate effects,
\code{\link{bayesIVgauss}} for estimation assuming a bivariate normal error distribution, \code{\link{am}} for estimation without bias correction; package \code{BayesX} for description of spatial effects related functions.
%% ~~objects to See Also as \code{\link{help}}, ~~~
}
\examples{
 \dontrun{
##################
# parametric model
  beta=1
  burnIn=10000
  numSamples=40000
  thin=40
  N=100
  rho=0.9
  R2=0.1
  k=1

  delta=  sqrt((12*R2)/((1-R2)*k))
  delta=matrix(rep(delta,k))
  Sigma=matrix(c(1,rho,rho,1),2,2)

  set.seed(1)

  z=matrix(runif(k*N),ncol=k)

  #Gaussian errors
    E=rmvnorm(N,c(0,0),sigma=Sigma)

    x=z\%*\%delta+E[,1]
    y=beta*x+E[,2]
    data=data.frame(x,y,z)

    first=x~z
    second=y~x
    bG1=am(form=second,numBurnIn=burnIn, numSamples=numSamples, seed=1,
            data=data, thin=thin,mgcv=FALSE,plotpaths=FALSE)
    bivG1=bayesIVgauss(first=first, second=second, data=data,
              numBurnIn=burnIn, numSamples=numSamples, thin=thin)
    bivDP1=bayesIV(first=first, second=second,
              numBurnIn=burnIn, numSamples=numSamples, thin=thin, data=data)

    #two-stage least squares regression for comparison:
    #  library(systemfit)
    #  tsls1=systemfit(second,"2SLS",inst=first[-2])
    #  summary(tsls1)

    summary(bivDP1)
    plotacf(bivDP1)
    plotpaths(bivDP1)
    plotpost(bivDP1,equation=2)
    par(mfrow=c(2,2))
    density(bivDP1)

  #lognormal errors
   set.seed(1)
    s=.6
    mu=c(0,0)
    root=chol(s*Sigma)
    ninterq=qnorm(.75)-qnorm(.25)
    error=matrix(rnorm(100000*2),ncol=2)
    error=t(t(error)+mu)
    E=t(t(exp(error))-exp(mu+.5*s*diag(Sigma)))
    lnNinterq=quantile(E[,1],prob=.75)-quantile(E[,1],prob=.25)
    error=matrix(rnorm(N*2),ncol=2)%*%root
    error=t(t(error)+mu)
    E=t(t(exp(error))-exp(mu+.5*s*diag(Sigma)))
    E[,1]=E[,1]*ninterq/lnNinterq
    E[,2]=E[,2]*ninterq/lnNinterq

    x=z\%*\%delta+E[,1]
    y=beta*x+E[,2]
    data=data.frame(x,y,z)


    first=x~z
    second=y~x
    bG2=am(form=second,numBurnIn=burnIn, numSamples=numSamples, data=data, thin=thin)
    bivG2=bayesIVgauss(first=first, second=second, data=data,
              numBurnIn=burnIn, numSamples=numSamples, thin=thin)
    bivDP2=bayesIV(first=first,second=second, data=data,
              numBurnIn=burnIn, numSamples=numSamples, thin=thin)

    #two-stage least squares regression for comparison:
    #  library(systemfit)
    #  tsls2=systemfit(second,"2SLS",inst=first[-2])
    #  summary(tsls2)

    plotacf(bivDP2)
    plotpaths(bivDP2)
    plotpost(bivDP2,equation=2)
    par(mfrow=c(2,2))
    density(bivDP2)

    #different color scheme
    par(mfrow=c(2,2))
    Lab.palette <- colorRampPalette(c("blue", "orange", "red"), space = "Lab")
    density(bivDP2,contour.col= Lab.palette(300),nlevels=25)


#################
# additive model
  center=function(x) return(x-mean(x))
  f2=function(x) (.5*(x^3+sin(pi*x^3))/0.2901875)
  f3=function(x) (-.5*(x+sin(pi*x^(2.5)))/0.3031512)
  f4=function(x) (-exp(-3*x) /0.2568223)
  f5=function(x) ((x^11*(10*(1-x))^6+10*(10*x)^3*(1-x))/387.5719)
  f6=function(x) (x^11*(10*(1-x))^6+10*(10*x)^3*(1-x)^10)/4.880719
  burnIn=1000
  numSamples=10000
  thin=10
  N=300
  R2=0.3#0.1

  delta2=1.2
  delta3=1

  delta1=sqrt((R2*(delta2^2+1))/(1-R2))

  set.seed(1)
  x2=runif(N)
  xu=runif(N)
  z1=runif(N)

  e1=rnorm(N)
  e2=rnorm(N)

  xe=delta1*f5(z1)+delta2*f4(xu)+ e1
  xe=(xe-min(xe))/(max(xe)-min(xe))
  y=f2(xe)+f6(x2)+delta3*f3(xu)+e2

  first=xe~s(z1)
  second=y~s(xe)+s(x2)

  ngrid=200
  predlist= list(seq(0,1,length.out=ngrid),seq(0,1,length.out=ngrid),
                  seq(0,1,length.out=ngrid))

  data=data.frame(y,xe,z1,x2,xu)
  bG3=am(form=second, data=data, numBurnIn=burnIn, numSamples=numSamples,
            thin=thin, x.pred=predlist, scbs=TRUE)
  bivG3=bayesIVgauss(first=first, second=second, data=data, x.pred=predlist,
            numBurnIn=burnIn, numSamples=numSamples, thin=thin, scbs=TRUE)
  bivDP3=bayesIV(first=first,second=second, data=data, x.pred=predlist,
            numBurnIn=burnIn, numSamples=numSamples, thin=thin, scbs=TRUE)
  predlist= list(seq(0,1,length.out=ngrid), seq(0,1,length.out=ngrid),
                 seq(0,1,length.out=ngrid), seq(0,1,length.out=ngrid),
                 seq(0,1,length.out=ngrid))
  bGfull3=am(form=y~s(xe)+s(x2)+s(xu), data=data, x.pred=predlist,
              numBurnIn=burnIn, numSamples=numSamples, thin=thin, scbs=TRUE)

   plot(bivDP3,equation=2,covariate=1,ask=FALSE,lwd=2)
  lines(bivG3$x.pred[[2]],bivG3$f.pred[[2]],col="brown",lwd=2)
  lines(bG3$x.pred[[1]],bG3$f.pred[[1]],col="lightblue",lwd=2)
  lines(bGfull3$x.pred[[1]],bGfull3$f.pred[[1]],col="red",lwd=2)
  legend("topright",
      c("BayesDPM","BayesIVgauss","Bayes univ.","mgcv","Bayes univ. full"),
      text.col=c("black","brown","lightblue","blue","red"),cex=1)

  par(mfrow=c(2,2))
  density(bivDP3)
  }
}