\name{plot.bayesIV}
\alias{plot.bayesIV}

\title{Plot nonparametric effects}

\description{
Plots nonparametric effects of a \code{bayesIV} object.
}

\usage{
plot(object,covariate=NULL,equation=NULL,center=TRUE,PLOT=TRUE,ask=dev.interactive(),
xlab=NULL,ylab=NULL,ylim=NULL,main=NULL,lwd=1, col="black",
mgcv=T,mgcv.col="blue", mgcv.lty="solid",mgcv.lwd=1,
ci=T, ci.lines=c(F,F), ci.lty = c("dotted","dotted"),ci.col=c(grey(0.80),grey(0.85)),
scb=T,scb.level=0.95,scb.method="TK", scb.lty="dashed",scb.lwd=1,
meanlevel=T,rug=T,jitter=T,...)
}

\arguments{
  \item{object}{a \code{bayesIV} object}
  \item{covariate}{position of covariate in the corresponding equation or variable name (in case of varying coefficients name after "by=" counts). If unspecified, all estimated curves (in equation as given by \code{equation}) are plotted.}
  \item{equation}{either 1 for first or 2 for second equation. If unspecified and if \code{covariate} is unspecified, all estimated curves of both equations are plotted.}
  \item{center}{TRUE in order to center simultaneous credible band around zero. Otherwise the intercept (and its variability) is included.}
  \item{PLOT}{if FALSE no plotting is performed}
  \item{ask}{if TRUE (and the R session is interactive) the user is asked for input, before a new figure is drawn.}
  \item{mgcv}{add \code{\link{mgcv}} estimate. Only available if \code{mgcv=TRUE} was set when estimating e.g. with \code{\link{bayesIV}}}
  \item{mgcv.col, mgcv.lty, mgcv.lwd}{color etc. of mgcv estimate}
  \item{ci}{if TRUE pointwise credible intervals are added.}
 \item{ci.lines}{ draw lines for pointwise CIs (vector: first 90\% CI, second 95\% CI)}
  \item{ci.lty}{ lty for pointwise CIs (vector: first 90\% CI, second 95\% CI)}
  \item{ci.col}{ color to fill pointwise CIs (vector: first 90\% CI, second 95\% CI); specify c("white","white") if you don't want the intervals to be filled, c(somecolor,"white") to only show the 90\% interval filled and c(somecolor,somecolor) if you only want to show the 95\% interval}
   \item{scb}{plot simultanous credible bands. Note that when estimating, \code{return.f=TRUE} has to be used}
  \item{scb.level}{level of simultaneous credible band}
  \item{scb.method}{method for simultaneous confidence band: "TK" (Thomas Kneib), "CC" (Ciprian Crainiceanu) or "JB" (Besag)}
  \item{scb.lty}{line type of simultaneous credible band}
  \item{scb.lwd}{line width of simultaneous credible band}
  \item{meanlevel}{ draw mean level in case of varying coefficients?}
  \item{rug}{if TRUE (default), adds a \code{\link{rug}} representation (1-d plot) of the data to the plot.}
  \item{jitter}{add a small amount of noise to the covariate in the \code{\link{rug}} representation.}
  \item{...}{arguments to be passed to \code{\link{plot}}}
}

\author{Manuel Wiesenfarth}
\references{
%Wiesenfarth, M. and Kneib, T. Bayesian Geoadditive Sample Selection Models...
}
\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
%package \code{sampleSelection} for frequentist sample selection models, package \code{BayesX} for description of spatial effects related functions.
}

\examples{
#see bayesIV()
}