\name{ssm}
\alias{ssm}

\title{ Fitting Bayesian Sample Selection Models }

\description{
With this function semiparametric sample selection models with possible geoadditive effects and
varying coefficients can be fitted. Inference is fully Bayesian. Nonparametric effects are modeled
with P-splines.
}

\usage{
ssm(selection, outcome, data, numBurnIn=2000, numSamples=20000, thin=20
, map.sel = NULL, graph.sel= NULL, map.out = NULL, graph.out= NULL
,contrasts.sel =NULL, contrasts.out =NULL
, numKnots=20, knotchoice="equidistant", degree=3, rw.order=2
, x.pred=NULL, fix.sigma12=F
, sigma2_1.start=10
, prior.sigma2_1=c(0.001,0.001), prior.sigma12=c(0,10)
, prior.tau=c(0.001,0.001), prior.tau.sp=c(0.001,0.001)
, seed=NULL
, record.npcoef=T, return.samples=T, return.f=T
, output="window"
, mgcv=F, add.mgcv=T, sampleSelection="n"
, plotnp=T, plotmaps=T, cols="hcl"
, plotacf=F,lag=50, plotpaths=F, plotpaths.style="separate"
, progressBar=T
, method="chol",...)
}

\arguments{
\item{selection, outcome}{The formulas of selection and outcome equation.
Specification of formulas is similar to package \code{mgcv}, with an additional possibility to fit spatial
    effects specified with \code{sp(.)} (smooth functions
    are specified with \code{s(.)} and varying coefficients with
    \code{s(., by=.)}). If nonparametric effects are included, an
    intercept is obligatory, otherwise exclusion of the intercept with
    -1 and +0, as usual.}
%%okay
\item{data}{A data frame containing the variables in the model.}
%%okay
\item{map.sel, map.out}{Specification either of a \code{.bnd} map file or of a bnd map object for spatial effects in the
    selection and outcome equation, respectively. If both equations include a spatial effect and only one map is
    specified, maps are assumed to be equal in both equations. See package BayesX for details on bnd files.}
%%okay
\item{graph.sel, graph.out}{Specification of a \code{.gra} graph files for spatial effects in the selection and outcome
    equation containing information of neighborhood structures. If only one graph file is specified, neighborhood structures are assumed to be equal in both equations.}

\item{contrasts.sel, contrasts.out}{Optional lists for the specification of contrasts in the case of catergorical covariates in selection and outcome equation, respectively. See \code{contrasts.arg} of code{\link{model.matrix.default}}.}
%%okay
\item{numBurnIn, numSamples}{Number of iterations in burn-in period that will be discarded and number of samples that will be used for estimation, respectively.}
%%okay
\item{thin}{Specify the degree of thinning. If 1, every sample after the burn-in period is used.}
%%okay
\item{numKnots}{Number of B-spline basis functions for nonparametric effects.}
%%okay
\item{knotchoice}{Specify the way knots are located. Either "equidistant" or "quantiles".}
%%okay
\item{degree}{Specifies the degree of the B-spline basis functions according to function bs() in package \code{splines}, default is 3 for cubic splines.}
\item{rw.order}{Specify the order of the random walk penalty matrices of smooth functions. }
%%okay
\item{x.pred}{(optional) List of vectors of covariates for which predicted function values are returned. }
%%genauer
\item{fix.sigma12}{Set covariance of the errors fixed at zero. If \code{TRUE}, results will correspond to separate univariate regressions, i.e. no sample selection model is fitted but the equations are modeled independently.}
%%okay
\item{sigma2_1.start}{ Starting value for the conditional variance sigma_{2|1}^2. Note that the starting values for the parametric effects equal to univariate regression estimates and all other starting values are set to 0.}
\item{prior.sigma2_1}{ Shape and scale parameters for the inverse Gamma prior on the covariance component sigma_{2|1}^2 (==sigma2_1). }
\item{prior.sigma12}{  Mean and variance of the Gaussian prior on the covariance sigma12 of the errors.}
\item{prior.tau}{shape and scale parameters for the inverse Gamma prior on the variance parameters which influence the smoothness of curves.}
\item{prior.tau.sp}{shape and scale parameters for the inverse Gamma prior on the variance parameters which influence the smoothness of spatial effects.}
%%priors nochmal anschauen
\item{seed}{Specify a seed for set.seed (optional).}
\item{record.npcoef}{  Record samples of the regression coefficients of nonparametric effects? Returned only if return.samples=TRUE.  }
\item{return.samples}{  Return matrices of samples of all estimates (including components of the covariance matrix, nonparametric (only if record.beta=T) and parametric estimates)? }
\item{return.f}{  Specify if fitted values for every smooth effect of each Gibbs iteration are to be returned. }
\item{output}{  Device on which plots are printed. Default is "window", otherwise pdf files are written with file name starting with what is given here.}
\item{mgcv}{  Specify if univariate estimation using package mgcv is included for comparison. Spatial effects are discarded.}
\item{add.mgcv}{  Add smooth functions estimated by \code{mgcv} to the plots of nonparametric effects. Doesn't work with varying coefficients.       }
\item{sampleSelection}{  Specify if Heckman-style estimation using package \code{sampleSelection} is conducted for comparison. Spatial effects are discarded
    and nonparametric effects are linearly estimated. Possible values
    "n" for no estimation, "2step" for Heckman's Two-step estimator,
    "ML" for Full Information Maximum Likelihood estimation and "both"
    for both of them.              }
\item{plotnp}{  Logical, if TRUE nonparametric effects are plotted  }
\item{plotmaps}{  Logical, if TRUE the maps(s) with spatial effect(s) is/are plotted}
\item{cols}{Color scheme to be employed for plotting spatial maps. Could be either a vector of colors or one out of the following pre-defined schemes: hcl, hsv, grey}
\item{plotacf}{  Plot autocorrelation functions? If nonparametric effects are included autocorrelation functions of the first and 10th knot are plotted, but only if record.beta==T.              }
\item{lag}{  Specify the lag of the autocorrelation functions.    }
\item{plotpaths}{  Logical, if TRUE the sampling paths are plotted  }
\item{progressBar}{Show txtProgressBar during simulation}
}

\details{The Gibbs sampling scheme involved is based on the joint posterior distribution marginalized over unobserved observations.
As usual in sample selection models, two equations described by the formulas \code{selection} and \code{outcome} are assumed.
The dependent variable in the selection equation determines if the dependent variable in the outcome equation is observed.
The response variable of the selection equation must be binary with 0 when the response of the outcome equation is unobserved and 1 if the
response variable of the outcome equation is observed. The response variable of the outcome equation is assumed to be Gaussian
when it is observed (when the response of the selection equation is 1).
In case of categorical independent variables, dummy variables must be constructed by the user, i.e. all independent variables are considered as metric.


%For comparison with univariate regression, linear estimates are given in columns "glm" and "lm"
%calculated by functions \code{glm(family="probit")} and \code{lm}, respectively (note that these
%estimates are calculated assuming all effects linear -- those specified nonparamtrically, too --
%and excluding possible spatial effects). If \code{mgcv==T}, parametric estimates calculated with
%function \code{gam} of package \code{mgcv} are given in the columns aside (nonparametric effects
%are calculated and plotted, but again excluding spatial effects).

For spatial effects a .gra file and a .bnd file are needed describing the neighborhood structure and boundary information. See package BayesX for details.

The output includes resulting estimates, as well as the corresponding credible intervals and
posterior probabilities.
For model selection the DIC (Deviance Information Criterion) which is based on the joint likelihood, is calculated.
It is recommended to choose large numbers of numBurnIn, numSamples and thin and to check sample autocorrelations due to a tendency of high sample autocorrelations of parameters in the selection equation.

}

\value{
\item{coefficients.sample, npcoef.sample, sigma12.sample, sigma2_1.sample,}{MCMC samples of the parameters after the burn-in period and after thinning.   }
\item{f.sample}{MCMC samples of the nonparametric functions.   }

\item{f, f.mgcv}{Function values evaluated at the designpoints for the sample selection model and separate univariate regressions using \code{mgcv:gam}.   }
\item{f.by}{Function values of functions with varying coefficients evaluated at the designpoints for the sample selection model   }
\item{ci.f}{The corresponding credible intervals.   }
\item{f.pred, f.mgcv.pred, ci.f.pred}{Predicted function values evaluated at points given by \code{x.pred} for the sample selection model and separate univariate regressions using \code{mgcv:gam} and the corresponding credible intervals.   }

\item{pcat80, pcat95}{For the visualisation of 80\% and 95\% posterior probabilities of geoadditive effects. Regions with significantly positive, insignificant and significantly negative values are coded with +1, 0 and -1, respectively.   }

\item{coefficients,coefficients.mgcv,coefficients.glm,coefficients.lm}{Coefficients  in the Bayesian sample selection model, separate univariate regressions obtained by \code{mgcv:gam} and separate univariate parametric regressions obtained by \code{glm} and \code{lm}.   }
\item{sd.coefficients,sd.coefficients.mgcv}{The standard deviations of the coefficients of parametric effects for the sample selection model and separate univariate regressions using \code{mgcv:gam}  }
\item{ci.coefficients}{ 90\% and 95\% credible intervals of the coefficients of parametric effects for the sample selection model  }
\item{npcoef,ci.npcoef}{coefficients and credible intervals of the coefficients of nonparametric effects  }


\item{cov, sigma12, sigma2_1, corr}{The covariance matrix of the errors, the associated covariance sigma12, conditional variance $sigma_{2|1}^2$ and correlation.}
\item{ci.sigma12, ci.sigma22, ci.corr}{90\% and 95\% credible intervals of the covariance, the error in the outcome equation and the correlation between the errors based on the respective quantiles of the samples. }
\item{sd.sigma12, sd.sigma22, sd.corr}{The respective standard deviations.}

\item{pvalues.onesided}{Onesided p values.   }
\item{pvalues.twosided}{Two-sided p values.   }
\item{DIC}{Deviance Information Criterion for model selection.}

\item{responses}{The responses}
\item{covariates}{The included covariates}
\item{x.pred}{   }
\item{fitted.values}{The fitted values.}

\item{formulas, numSplines, numSpatial, numBurnIn, numSamples, thin}{Arguments given in the function call.   }
}

\author{Manuel Wiesenfarth}
\references{
Wiesenfarth, M., Kneib, T. (2010). Bayesian Geoadditive Sample Selection Models.
Journal of the Royal Statistical Society: Series C (Applied Statistics), 59(3): 381-404.
}
\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
package \code{sampleSelection} for frequentist sample selection models, package \code{BayesX} for description of spatial effects related functions.
}

\examples{
 \dontrun{
library(mvtnorm)
n=100
x1 = rnorm(n=n, mean=0.5, sd=0.6)
x2 = runif(n=n, min=0, max=1)
e=rmvnorm(n=n, mean=c(0,0),sigma=matrix(c(1,0.5,0.5,1),ncol=2))

l = 2*x1 + e[,1]
z=numeric(n)
z[l>0]=1
z[l<=0]=0
y = 1.5*x1 + sin(x2) + e[,2]
y[z==0]=0
data=data.frame(cbind(y,z,x1,x2))

selection      = z~x1
outcome      = y~x1 +s(x2)
sim=ssm(selection=selection, outcome=outcome, data=data,numBurnIn=10000,numSamples=40000,thin=40, numKnots=20) 
sim
summary(sim)
plot(sim,1,2)
plotacf(sim)
plotpaths(sim)
}
}